@@ -26,17 +26,17 @@ import org.apache.commons.compress.archivers.zip.ZipUtil;
 
 /**
  * An entry in an ARJ archive.
- * 
+ *
  * @NotThreadSafe
  * @since 1.6
  */
 public class ArjArchiveEntry implements ArchiveEntry {
     private final LocalFileHeader localFileHeader;
-    
+
     public ArjArchiveEntry() {
         localFileHeader = new LocalFileHeader();
     }
-    
+
     ArjArchiveEntry(final LocalFileHeader localFileHeader) {
         this.localFileHeader = localFileHeader;
     }
@@ -44,15 +44,17 @@ public class ArjArchiveEntry implements ArchiveEntry {
     /**
      * Get this entry's name.
      *
+     * <p>This method returns the raw name as it is stored inside of the archive.</p>
+     *
      * @return This entry's name.
      */
+    @Override
     public String getName() {
         if ((localFileHeader.arjFlags & LocalFileHeader.Flags.PATHSYM) != 0) {
             return localFileHeader.name.replaceAll("/",
                     Matcher.quoteReplacement(File.separator));
-        } else {
-            return localFileHeader.name;
         }
+        return localFileHeader.name;
     }
 
     /**
@@ -60,11 +62,16 @@ public class ArjArchiveEntry implements ArchiveEntry {
      *
      * @return This entry's file size.
      */
+    @Override
     public long getSize() {
         return localFileHeader.originalSize;
     }
 
-    /** True if the entry refers to a directory */
+    /** True if the entry refers to a directory.
+     *
+     * @return True if the entry refers to a directory
+     */
+    @Override
     public boolean isDirectory() {
         return localFileHeader.fileType == LocalFileHeader.FileTypes.DIRECTORY;
     }
@@ -81,9 +88,12 @@ public class ArjArchiveEntry implements ArchiveEntry {
      * regardless of timezone if the archive has been created on a
      * non-Unix system and a time taking the current timezone into
      * account if the archive has beeen created on Unix.</p>
+     *
+     * @return the last modified date
      */
+    @Override
     public Date getLastModifiedDate() {
-        long ts = isHostOsUnix() ? localFileHeader.dateTimeModified * 1000l
+        final long ts = isHostOsUnix() ? localFileHeader.dateTimeModified * 1000L
             : ZipUtil.dosToJavaTime(0xFFFFFFFFL & localFileHeader.dateTimeModified);
         return new Date(ts);
     }
@@ -92,6 +102,8 @@ public class ArjArchiveEntry implements ArchiveEntry {
      * File mode of this entry.
      *
      * <p>The format depends on the host os that created the entry.</p>
+     *
+     * @return the file mode
      */
     public int getMode() {
         return localFileHeader.fileAccessMode;
@@ -101,6 +113,8 @@ public class ArjArchiveEntry implements ArchiveEntry {
      * File mode of this entry as Unix stat value.
      *
      * <p>Will only be non-zero of the host os was UNIX.
+     *
+     * @return the Unix mode
      */
     public int getUnixMode() {
         return isHostOsUnix() ? getMode() : 0;
@@ -109,6 +123,7 @@ public class ArjArchiveEntry implements ArchiveEntry {
     /**
      * The operating system the archive has been created on.
      * @see HostOs
+     * @return the host OS code
      */
     public int getHostOs() {
         return localFileHeader.hostOS;
@@ -117,11 +132,17 @@ public class ArjArchiveEntry implements ArchiveEntry {
     /**
      * Is the operating system the archive has been created on one
      * that is considered a UNIX OS by arj?
+     * @return whether the operating system the archive has been
+     * created on is considered a UNIX OS by arj
      */
     public boolean isHostOsUnix() {
         return getHostOs() == HostOs.UNIX || getHostOs() == HostOs.NEXT;
     }
 
+    int getMethod() {
+        return localFileHeader.method;
+    }
+
     /**
      * The known values for HostOs.
      */
@@ -139,5 +160,5 @@ public class ArjArchiveEntry implements ArchiveEntry {
         public static final int WIN95 = 10;
         public static final int WIN32 = 11;
     }
-    
+
 }
