@@ -22,6 +22,7 @@ import java.io.File;
 import java.io.IOException;
 import java.io.OutputStream;
 import java.nio.ByteBuffer;
+import java.util.Arrays;
 import java.util.HashMap;
 
 import org.apache.commons.compress.archivers.ArchiveEntry;
@@ -32,7 +33,7 @@ import org.apache.commons.compress.utils.ArchiveUtils;
 import org.apache.commons.compress.utils.CharsetNames;
 
 /**
- * CPIOArchiveOutputStream is a stream for writing CPIO streams. All formats of
+ * CpioArchiveOutputStream is a stream for writing CPIO streams. All formats of
  * CPIO are supported (old ASCII, old binary, new portable format and the new
  * portable format with CRC).
  *
@@ -56,9 +57,9 @@ import org.apache.commons.compress.utils.CharsetNames;
  * </pre>
  *
  * <p>Note: This implementation should be compatible to cpio 2.5</p>
- * 
+ *
  * <p>This class uses mutable fields and is not considered threadsafe.</p>
- * 
+ *
  * <p>based on code from the jRPM project (jrpm.sourceforge.net)</p>
  */
 public class CpioArchiveOutputStream extends ArchiveOutputStream implements
@@ -101,7 +102,7 @@ public class CpioArchiveOutputStream extends ArchiveOutputStream implements
      * Construct the cpio output stream with a specified format, a
      * blocksize of {@link CpioConstants#BLOCK_SIZE BLOCK_SIZE} and
      * using ASCII as the file name encoding.
-     * 
+     *
      * @param out
      *            The cpio stream
      * @param format
@@ -114,25 +115,25 @@ public class CpioArchiveOutputStream extends ArchiveOutputStream implements
     /**
      * Construct the cpio output stream with a specified format using
      * ASCII as the file name encoding.
-     * 
+     *
      * @param out
      *            The cpio stream
      * @param format
      *            The format of the stream
      * @param blockSize
      *            The block size of the archive.
-     * 
+     *
      * @since 1.1
      */
     public CpioArchiveOutputStream(final OutputStream out, final short format,
                                    final int blockSize) {
         this(out, format, blockSize, CharsetNames.US_ASCII);
-    }        
+    }
 
     /**
      * Construct the cpio output stream with a specified format using
      * ASCII as the file name encoding.
-     * 
+     *
      * @param out
      *            The cpio stream
      * @param format
@@ -142,7 +143,7 @@ public class CpioArchiveOutputStream extends ArchiveOutputStream implements
      * @param encoding
      *            The encoding of file names to write - use null for
      *            the platform's default.
-     * 
+     *
      * @since 1.6
      */
     public CpioArchiveOutputStream(final OutputStream out, final short format,
@@ -167,7 +168,7 @@ public class CpioArchiveOutputStream extends ArchiveOutputStream implements
     /**
      * Construct the cpio output stream. The format for this CPIO stream is the
      * "new" format using ASCII encoding for file names
-     * 
+     *
      * @param out
      *            The cpio stream
      */
@@ -178,7 +179,7 @@ public class CpioArchiveOutputStream extends ArchiveOutputStream implements
     /**
      * Construct the cpio output stream. The format for this CPIO stream is the
      * "new" format.
-     * 
+     *
      * @param out
      *            The cpio stream
      * @param encoding
@@ -192,7 +193,7 @@ public class CpioArchiveOutputStream extends ArchiveOutputStream implements
 
     /**
      * Check to make sure that this stream has not been closed
-     * 
+     *
      * @throws IOException
      *             if the stream is already closed
      */
@@ -208,7 +209,7 @@ public class CpioArchiveOutputStream extends ArchiveOutputStream implements
      * current time will be used if the entry has no set modification time and
      * the default header format will be used if no other format is specified in
      * the entry.
-     * 
+     *
      * @param entry
      *            the CPIO cpioEntry to be written
      * @throws IOException
@@ -299,10 +300,11 @@ public class CpioArchiveOutputStream extends ArchiveOutputStream implements
         writeAsciiLong(devMin, 8, 16);
         writeAsciiLong(entry.getRemoteDeviceMaj(), 8, 16);
         writeAsciiLong(entry.getRemoteDeviceMin(), 8, 16);
-        writeAsciiLong(entry.getName().length() + 1l, 8, 16);
+        byte[] name = encode(entry.getName());
+        writeAsciiLong(name.length + 1L, 8, 16);
         writeAsciiLong(entry.getChksum(), 8, 16);
-        writeCString(entry.getName());
-        pad(entry.getHeaderPadCount());
+        writeCString(name);
+        pad(entry.getHeaderPadCount(name.length));
     }
 
     private void writeOldAsciiEntry(final CpioArchiveEntry entry)
@@ -330,9 +332,10 @@ public class CpioArchiveOutputStream extends ArchiveOutputStream implements
         writeAsciiLong(entry.getNumberOfLinks(), 6, 8);
         writeAsciiLong(entry.getRemoteDevice(), 6, 8);
         writeAsciiLong(entry.getTime(), 11, 8);
-        writeAsciiLong(entry.getName().length() + 1l, 6, 8);
+        byte[] name = encode(entry.getName());
+        writeAsciiLong(name.length + 1L, 6, 8);
         writeAsciiLong(entry.getSize(), 11, 8);
-        writeCString(entry.getName());
+        writeCString(name);
     }
 
     private void writeOldBinaryEntry(final CpioArchiveEntry entry,
@@ -360,14 +363,15 @@ public class CpioArchiveOutputStream extends ArchiveOutputStream implements
         writeBinaryLong(entry.getNumberOfLinks(), 2, swapHalfWord);
         writeBinaryLong(entry.getRemoteDevice(), 2, swapHalfWord);
         writeBinaryLong(entry.getTime(), 4, swapHalfWord);
-        writeBinaryLong(entry.getName().length() + 1l, 2, swapHalfWord);
+        byte[] name = encode(entry.getName());
+        writeBinaryLong(name.length + 1L, 2, swapHalfWord);
         writeBinaryLong(entry.getSize(), 4, swapHalfWord);
-        writeCString(entry.getName());
-        pad(entry.getHeaderPadCount());
+        writeCString(name);
+        pad(entry.getHeaderPadCount(name.length));
     }
 
     /*(non-Javadoc)
-     * 
+     *
      * @see
      * org.apache.commons.compress.archivers.ArchiveOutputStream#closeArchiveEntry
      * ()
@@ -402,7 +406,7 @@ public class CpioArchiveOutputStream extends ArchiveOutputStream implements
     /**
      * Writes an array of bytes to the current CPIO entry data. This method will
      * block until all the bytes are written.
-     * 
+     *
      * @param b
      *            the data to be written
      * @param off
@@ -434,6 +438,7 @@ public class CpioArchiveOutputStream extends ArchiveOutputStream implements
         if (this.entry.getFormat() == FORMAT_NEW_CRC) {
             for (int pos = 0; pos < len; pos++) {
                 this.crc += b[pos] & 0xFF;
+                this.crc &= 0xFFFFFFFFL;
             }
         }
         count(len);
@@ -443,7 +448,7 @@ public class CpioArchiveOutputStream extends ArchiveOutputStream implements
      * Finishes writing the contents of the CPIO output stream without closing
      * the underlying stream. Use this method when applying multiple filters in
      * succession to the same output stream.
-     * 
+     *
      * @throws IOException
      *             if an I/O exception has occurred or if a CPIO file error has
      *             occurred
@@ -474,20 +479,22 @@ public class CpioArchiveOutputStream extends ArchiveOutputStream implements
 
     /**
      * Closes the CPIO output stream as well as the stream being filtered.
-     * 
+     *
      * @throws IOException
      *             if an I/O error has occurred or if a CPIO file error has
      *             occurred
      */
     @Override
     public void close() throws IOException {
-        if(!finished) {
-            finish();
-        }
-
-        if (!this.closed) {
-            out.close();
-            this.closed = true;
+        try {
+            if (!finished) {
+                finish();
+            }
+        } finally {
+            if (!this.closed) {
+                out.close();
+                this.closed = true;
+            }
         }
     }
 
@@ -533,21 +540,32 @@ public class CpioArchiveOutputStream extends ArchiveOutputStream implements
     }
 
     /**
-     * Writes an ASCII string to the stream followed by \0
+     * Encodes the given string using the configured encoding.
+     *
+     * @param str the String to write
+     * @throws IOException if the string couldn't be written
+     * @return result of encoding the string
+     */
+    private byte[] encode(final String str) throws IOException {
+        final ByteBuffer buf = zipEncoding.encode(str);
+        final int len = buf.limit() - buf.position();
+        return Arrays.copyOfRange(buf.array(), buf.arrayOffset(), buf.arrayOffset() + len);
+    }
+
+    /**
+     * Writes an encoded string to the stream followed by \0
      * @param str the String to write
      * @throws IOException if the string couldn't be written
      */
-    private void writeCString(final String str) throws IOException {
-        final ByteBuffer buf = zipEncoding.encode(str);
-        final int len = buf.limit() - buf.position();
-        out.write(buf.array(), buf.arrayOffset(), len);
+    private void writeCString(byte[] str) throws IOException {
+        out.write(str);
         out.write('\0');
-        count(len + 1);
+        count(str.length + 1);
     }
 
     /**
      * Creates a new ArchiveEntry. The entryName must be an ASCII encoded string.
-     * 
+     *
      * @see org.apache.commons.compress.archivers.ArchiveOutputStream#createArchiveEntry(java.io.File, java.lang.String)
      */
     @Override
