@@ -22,6 +22,7 @@ import java.io.File;
 import java.io.IOException;
 import java.io.OutputStream;
 import java.io.StringWriter;
+import java.io.UnsupportedEncodingException;
 import java.nio.ByteBuffer;
 import java.util.Arrays;
 import java.util.Date;
@@ -33,56 +34,84 @@ import org.apache.commons.compress.archivers.zip.ZipEncoding;
 import org.apache.commons.compress.archivers.zip.ZipEncodingHelper;
 import org.apache.commons.compress.utils.CharsetNames;
 import org.apache.commons.compress.utils.CountingOutputStream;
+import org.apache.commons.compress.utils.FixedLengthBlockOutputStream;
 
 /**
- * The TarOutputStream writes a UNIX tar archive as an OutputStream.
- * Methods are provided to put entries, and then write their contents
- * by writing to this stream using write().
+ * The TarOutputStream writes a UNIX tar archive as an OutputStream. Methods are provided to put
+ * entries, and then write their contents by writing to this stream using write().
+ *
+ * <p>tar archives consist of a sequence of records of 512 bytes each
+ * that are grouped into blocks. Prior to Apache Commons Compress 1.14
+ * it has been possible to configure a record size different from 512
+ * bytes and arbitrary block sizes. Starting with Compress 1.15 512 is
+ * the only valid option for the record size and the block size must
+ * be a multiple of 512. Also the default block size changed from
+ * 10240 bytes prior to Compress 1.15 to 512 bytes with Compress
+ * 1.15.</p>
+ *
  * @NotThreadSafe
  */
 public class TarArchiveOutputStream extends ArchiveOutputStream {
-    /** Fail if a long file name is required in the archive. */
+
+    /**
+     * Fail if a long file name is required in the archive.
+     */
     public static final int LONGFILE_ERROR = 0;
 
-    /** Long paths will be truncated in the archive. */
+    /**
+     * Long paths will be truncated in the archive.
+     */
     public static final int LONGFILE_TRUNCATE = 1;
 
-    /** GNU tar extensions are used to store long file names in the archive. */
+    /**
+     * GNU tar extensions are used to store long file names in the archive.
+     */
     public static final int LONGFILE_GNU = 2;
 
-    /** POSIX/PAX extensions are used to store long file names in the archive. */
+    /**
+     * POSIX/PAX extensions are used to store long file names in the archive.
+     */
     public static final int LONGFILE_POSIX = 3;
 
-    /** Fail if a big number (e.g. size &gt; 8GiB) is required in the archive. */
+    /**
+     * Fail if a big number (e.g. size &gt; 8GiB) is required in the archive.
+     */
     public static final int BIGNUMBER_ERROR = 0;
 
-    /** star/GNU tar/BSD tar extensions are used to store big number in the archive. */
+    /**
+     * star/GNU tar/BSD tar extensions are used to store big number in the archive.
+     */
     public static final int BIGNUMBER_STAR = 1;
 
-    /** POSIX/PAX extensions are used to store big numbers in the archive. */
+    /**
+     * POSIX/PAX extensions are used to store big numbers in the archive.
+     */
     public static final int BIGNUMBER_POSIX = 2;
+    private static final int RECORD_SIZE = 512;
 
-    private long      currSize;
-    private String    currName;
-    private long      currBytes;
-    private final byte[]    recordBuf;
-    private int       assemLen;
-    private final byte[]    assemBuf;
-    private int       longFileMode = LONGFILE_ERROR;
-    private int       bigNumberMode = BIGNUMBER_ERROR;
+    private long currSize;
+    private String currName;
+    private long currBytes;
+    private final byte[] recordBuf;
+    private int longFileMode = LONGFILE_ERROR;
+    private int bigNumberMode = BIGNUMBER_ERROR;
     private int recordsWritten;
     private final int recordsPerBlock;
-    private final int recordSize;
 
     private boolean closed = false;
 
-    /** Indicates if putArchiveEntry has been called without closeArchiveEntry */
+    /**
+     * Indicates if putArchiveEntry has been called without closeArchiveEntry
+     */
     private boolean haveUnclosedEntry = false;
 
-    /** indicates if this archive is finished */
+    /**
+     * indicates if this archive is finished
+     */
     private boolean finished = false;
 
-    private final OutputStream out;
+    private final FixedLengthBlockOutputStream out;
+    private final CountingOutputStream countingOut;
 
     private final ZipEncoding zipEncoding;
 
@@ -93,81 +122,114 @@ public class TarArchiveOutputStream extends ArchiveOutputStream {
     private static final ZipEncoding ASCII =
         ZipEncodingHelper.getZipEncoding("ASCII");
 
+    private static final int BLOCK_SIZE_UNSPECIFIED = -511;
+
     /**
-     * Constructor for TarInputStream.
+     * Constructor for TarArchiveOutputStream.
+     *
+     * <p>Uses a block size of 512 bytes.</p>
+     *
      * @param os the output stream to use
      */
     public TarArchiveOutputStream(final OutputStream os) {
-        this(os, TarConstants.DEFAULT_BLKSIZE, TarConstants.DEFAULT_RCDSIZE);
+        this(os, BLOCK_SIZE_UNSPECIFIED);
     }
 
     /**
-     * Constructor for TarInputStream.
+     * Constructor for TarArchiveOutputStream.
+     *
+     * <p>Uses a block size of 512 bytes.</p>
+     *
      * @param os the output stream to use
      * @param encoding name of the encoding to use for file names
      * @since 1.4
      */
     public TarArchiveOutputStream(final OutputStream os, final String encoding) {
-        this(os, TarConstants.DEFAULT_BLKSIZE, TarConstants.DEFAULT_RCDSIZE, encoding);
+        this(os, BLOCK_SIZE_UNSPECIFIED, encoding);
     }
 
     /**
-     * Constructor for TarInputStream.
+     * Constructor for TarArchiveOutputStream.
+     *
      * @param os the output stream to use
-     * @param blockSize the block size to use
+     * @param blockSize the block size to use. Must be a multiple of 512 bytes.
      */
     public TarArchiveOutputStream(final OutputStream os, final int blockSize) {
-        this(os, blockSize, TarConstants.DEFAULT_RCDSIZE);
+        this(os, blockSize, null);
     }
 
+
     /**
-     * Constructor for TarInputStream.
+     * Constructor for TarArchiveOutputStream.
+     *
      * @param os the output stream to use
      * @param blockSize the block size to use
-     * @param encoding name of the encoding to use for file names
-     * @since 1.4
+     * @param recordSize the record size to use. Must be 512 bytes.
+     * @deprecated recordSize must always be 512 bytes. An IllegalArgumentException will be thrown
+     * if any other value is used
      */
+    @Deprecated
     public TarArchiveOutputStream(final OutputStream os, final int blockSize,
-                                  final String encoding) {
-        this(os, blockSize, TarConstants.DEFAULT_RCDSIZE, encoding);
-    }
-
-    /**
-     * Constructor for TarInputStream.
-     * @param os the output stream to use
-     * @param blockSize the block size to use
-     * @param recordSize the record size to use
-     */
-    public TarArchiveOutputStream(final OutputStream os, final int blockSize, final int recordSize) {
+        final int recordSize) {
         this(os, blockSize, recordSize, null);
     }
 
     /**
-     * Constructor for TarInputStream.
+     * Constructor for TarArchiveOutputStream.
+     *
      * @param os the output stream to use
-     * @param blockSize the block size to use
-     * @param recordSize the record size to use
+     * @param blockSize the block size to use . Must be a multiple of 512 bytes.
+     * @param recordSize the record size to use. Must be 512 bytes.
+     * @param encoding name of the encoding to use for file names
+     * @since 1.4
+     * @deprecated recordSize must always be 512 bytes. An IllegalArgumentException will be thrown
+     * if any other value is used.
+     */
+    @Deprecated
+    public TarArchiveOutputStream(final OutputStream os, final int blockSize,
+        final int recordSize, final String encoding) {
+        this(os, blockSize, encoding);
+        if (recordSize != RECORD_SIZE) {
+            throw new IllegalArgumentException(
+                "Tar record size must always be 512 bytes. Attempt to set size of " + recordSize);
+        }
+
+    }
+
+    /**
+     * Constructor for TarArchiveOutputStream.
+     *
+     * @param os the output stream to use
+     * @param blockSize the block size to use. Must be a multiple of 512 bytes.
      * @param encoding name of the encoding to use for file names
      * @since 1.4
      */
     public TarArchiveOutputStream(final OutputStream os, final int blockSize,
-                                  final int recordSize, final String encoding) {
-        out = new CountingOutputStream(os);
+        final String encoding) {
+        int realBlockSize;
+        if (BLOCK_SIZE_UNSPECIFIED == blockSize) {
+            realBlockSize = RECORD_SIZE;
+        } else {
+            realBlockSize = blockSize;
+        }
+
+        if (realBlockSize <=0 || realBlockSize % RECORD_SIZE != 0) {
+            throw new IllegalArgumentException("Block size must be a multiple of 512 bytes. Attempt to use set size of " + blockSize);
+        }
+        out = new FixedLengthBlockOutputStream(countingOut = new CountingOutputStream(os),
+                                               RECORD_SIZE);
         this.encoding = encoding;
         this.zipEncoding = ZipEncodingHelper.getZipEncoding(encoding);
 
-        this.assemLen = 0;
-        this.assemBuf = new byte[recordSize];
-        this.recordBuf = new byte[recordSize];
-        this.recordSize = recordSize;
-        this.recordsPerBlock = blockSize / recordSize;
+        this.recordBuf = new byte[RECORD_SIZE];
+        this.recordsPerBlock = realBlockSize / RECORD_SIZE;
     }
 
     /**
-     * Set the long file mode.
-     * This can be LONGFILE_ERROR(0), LONGFILE_TRUNCATE(1) or LONGFILE_GNU(2).
-     * This specifies the treatment of long file names (names &gt;= TarConstants.NAMELEN).
-     * Default is LONGFILE_ERROR.
+     * Set the long file mode. This can be LONGFILE_ERROR(0), LONGFILE_TRUNCATE(1) or
+     * LONGFILE_GNU(2). This specifies the treatment of long file names (names &gt;=
+     * TarConstants.NAMELEN). Default is LONGFILE_ERROR.
+     *
      * @param longFileMode the mode to use
      */
     public void setLongFileMode(final int longFileMode) {
@@ -175,10 +237,11 @@ public class TarArchiveOutputStream extends ArchiveOutputStream {
     }
 
     /**
-     * Set the big number mode.
-     * This can be BIGNUMBER_ERROR(0), BIGNUMBER_POSIX(1) or BIGNUMBER_STAR(2).
-     * This specifies the treatment of big files (sizes &gt; TarConstants.MAXSIZE) and other numeric values to big to fit into a traditional tar header.
+     * Set the big number mode. This can be BIGNUMBER_ERROR(0), BIGNUMBER_POSIX(1) or
+     * BIGNUMBER_STAR(2). This specifies the treatment of big files (sizes &gt;
+     * TarConstants.MAXSIZE) and other numeric values to big to fit into a traditional tar header.
      * Default is BIGNUMBER_ERROR.
+     *
      * @param bigNumberMode the mode to use
      * @since 1.4
      */
@@ -188,8 +251,9 @@ public class TarArchiveOutputStream extends ArchiveOutputStream {
 
     /**
      * Whether to add a PAX extension header for non-ASCII file names.
-     * @since 1.4
+     *
      * @param b whether to add a PAX extension header for non-ASCII file names.
+     * @since 1.4
      */
     public void setAddPaxHeadersForNonAsciiNames(final boolean b) {
         addPaxHeadersForNonAsciiNames = b;
@@ -203,16 +267,16 @@ public class TarArchiveOutputStream extends ArchiveOutputStream {
 
     @Override
     public long getBytesWritten() {
-        return ((CountingOutputStream) out).getBytesWritten();
+        return countingOut.getBytesWritten();
     }
 
     /**
      * Ends the TAR archive without closing the underlying OutputStream.
-     * 
+     *
      * An archive consists of a series of file entries terminated by an
-     * end-of-archive entry, which consists of two 512 blocks of zero bytes. 
+     * end-of-archive entry, which consists of two 512 blocks of zero bytes.
      * POSIX.1 requires two EOF records, like some other implementations.
-     * 
+     *
      * @throws IOException on error
      */
     @Override
@@ -222,7 +286,7 @@ public class TarArchiveOutputStream extends ArchiveOutputStream {
         }
 
         if (haveUnclosedEntry) {
-            throw new IOException("This archives contains unclosed entries.");
+            throw new IOException("This archive contains unclosed entries.");
         }
         writeEOFRecord();
         writeEOFRecord();
@@ -233,17 +297,20 @@ public class TarArchiveOutputStream extends ArchiveOutputStream {
 
     /**
      * Closes the underlying OutputStream.
+     *
      * @throws IOException on error
      */
     @Override
     public void close() throws IOException {
-        if (!finished) {
-            finish();
-        }
-
-        if (!closed) {
-            out.close();
-            closed = true;
+        try {
+            if (!finished) {
+                finish();
+            }
+        } finally {
+            if (!closed) {
+                out.close();
+                closed = true;
+            }
         }
     }
 
@@ -251,19 +318,19 @@ public class TarArchiveOutputStream extends ArchiveOutputStream {
      * Get the record size being used by this stream's TarBuffer.
      *
      * @return The TarBuffer record size.
+     * @deprecated
      */
+    @Deprecated
     public int getRecordSize() {
-        return this.recordSize;
+        return RECORD_SIZE;
     }
 
     /**
-     * Put an entry on the output stream. This writes the entry's
-     * header record and positions the output stream for writing
-     * the contents of the entry. Once this method is called, the
-     * stream is ready for calls to write() to write the entry's
-     * contents. Once the contents are written, closeArchiveEntry()
-     * <B>MUST</B> be called to ensure that all buffered data
-     * is completely written to the output stream.
+     * Put an entry on the output stream. This writes the entry's header record and positions the
+     * output stream for writing the contents of the entry. Once this method is called, the stream
+     * is ready for calls to write() to write the entry's contents. Once the contents are written,
+     * closeArchiveEntry() <B>MUST</B> be called to ensure that all buffered data is completely
+     * written to the output stream.
      *
      * @param archiveEntry The TarEntry to be written to the archive.
      * @throws IOException on error
@@ -275,60 +342,70 @@ public class TarArchiveOutputStream extends ArchiveOutputStream {
             throw new IOException("Stream has already been finished");
         }
         final TarArchiveEntry entry = (TarArchiveEntry) archiveEntry;
-        final Map<String, String> paxHeaders = new HashMap<>();
-        final String entryName = entry.getName();
-        final boolean paxHeaderContainsPath = handleLongName(entry, entryName, paxHeaders, "path",
-                                                       TarConstants.LF_GNUTYPE_LONGNAME, "file name");
-
-        final String linkName = entry.getLinkName();
-        final boolean paxHeaderContainsLinkPath = linkName != null && linkName.length() > 0
-            && handleLongName(entry, linkName, paxHeaders, "linkpath",
-                              TarConstants.LF_GNUTYPE_LONGLINK, "link name");
-
-        if (bigNumberMode == BIGNUMBER_POSIX) {
-            addPaxHeadersForBigNumbers(paxHeaders, entry);
-        } else if (bigNumberMode != BIGNUMBER_STAR) {
-            failForBigNumbers(entry);
-        }
-
-        if (addPaxHeadersForNonAsciiNames && !paxHeaderContainsPath
-            && !ASCII.canEncode(entryName)) {
-            paxHeaders.put("path", entryName);
-        }
-
-        if (addPaxHeadersForNonAsciiNames && !paxHeaderContainsLinkPath
-            && (entry.isLink() || entry.isSymbolicLink())
-            && !ASCII.canEncode(linkName)) {
-            paxHeaders.put("linkpath", linkName);
-        }
-
-        if (paxHeaders.size() > 0) {
-            writePaxHeaders(entry, entryName, paxHeaders);
-        }
-
-        entry.writeEntryHeader(recordBuf, zipEncoding,
-                               bigNumberMode == BIGNUMBER_STAR);
-        writeRecord(recordBuf);
-
-        currBytes = 0;
-
-        if (entry.isDirectory()) {
-            currSize = 0;
+        if (entry.isGlobalPaxHeader()) {
+            final byte[] data = encodeExtendedPaxHeadersContents(entry.getExtraPaxHeaders());
+            entry.setSize(data.length);
+            entry.writeEntryHeader(recordBuf, zipEncoding, bigNumberMode == BIGNUMBER_STAR);
+            writeRecord(recordBuf);
+            currSize= entry.getSize();
+            currBytes = 0;
+            this.haveUnclosedEntry = true;
+            write(data);
+            closeArchiveEntry();
         } else {
-            currSize = entry.getSize();
+            final Map<String, String> paxHeaders = new HashMap<>();
+            final String entryName = entry.getName();
+            final boolean paxHeaderContainsPath = handleLongName(entry, entryName, paxHeaders, "path",
+                TarConstants.LF_GNUTYPE_LONGNAME, "file name");
+
+            final String linkName = entry.getLinkName();
+            final boolean paxHeaderContainsLinkPath = linkName != null && linkName.length() > 0
+                && handleLongName(entry, linkName, paxHeaders, "linkpath",
+                TarConstants.LF_GNUTYPE_LONGLINK, "link name");
+
+            if (bigNumberMode == BIGNUMBER_POSIX) {
+                addPaxHeadersForBigNumbers(paxHeaders, entry);
+            } else if (bigNumberMode != BIGNUMBER_STAR) {
+                failForBigNumbers(entry);
+            }
+
+            if (addPaxHeadersForNonAsciiNames && !paxHeaderContainsPath
+                && !ASCII.canEncode(entryName)) {
+                paxHeaders.put("path", entryName);
+            }
+
+            if (addPaxHeadersForNonAsciiNames && !paxHeaderContainsLinkPath
+                && (entry.isLink() || entry.isSymbolicLink())
+                && !ASCII.canEncode(linkName)) {
+                paxHeaders.put("linkpath", linkName);
+            }
+            paxHeaders.putAll(entry.getExtraPaxHeaders());
+
+            if (paxHeaders.size() > 0) {
+                writePaxHeaders(entry, entryName, paxHeaders);
+            }
+
+            entry.writeEntryHeader(recordBuf, zipEncoding, bigNumberMode == BIGNUMBER_STAR);
+            writeRecord(recordBuf);
+
+            currBytes = 0;
+
+            if (entry.isDirectory()) {
+                currSize = 0;
+            } else {
+                currSize = entry.getSize();
+            }
+            currName = entryName;
+            haveUnclosedEntry = true;
         }
-        currName = entryName;
-        haveUnclosedEntry = true;
     }
 
     /**
-     * Close an entry. This method MUST be called for all file
-     * entries that contain data. The reason is that we must
-     * buffer data written to the stream in order to satisfy
-     * the buffer's record based writes. Thus, there may be
-     * data fragments still being assembled that must be written
-     * to the output stream before this entry is closed and the
-     * next entry written.
+     * Close an entry. This method MUST be called for all file entries that contain data. The reason
+     * is that we must buffer data written to the stream in order to satisfy the buffer's record
+     * based writes. Thus, there may be data fragments still being assembled that must be written to
+     * the output stream before this entry is closed and the next entry written.
+     *
      * @throws IOException on error
      */
     @Override
@@ -336,37 +413,27 @@ public class TarArchiveOutputStream extends ArchiveOutputStream {
         if (finished) {
             throw new IOException("Stream has already been finished");
         }
-        if (!haveUnclosedEntry){
+        if (!haveUnclosedEntry) {
             throw new IOException("No current entry to close");
         }
-        if (assemLen > 0) {
-            for (int i = assemLen; i < assemBuf.length; ++i) {
-                assemBuf[i] = 0;
-            }
-
-            writeRecord(assemBuf);
-
-            currBytes += assemLen;
-            assemLen = 0;
-        }
-
+        out.flushBlock();
         if (currBytes < currSize) {
             throw new IOException("entry '" + currName + "' closed at '"
-                                  + currBytes
-                                  + "' before the '" + currSize
-                                  + "' bytes specified in the header were written");
+                + currBytes
+                + "' before the '" + currSize
+                + "' bytes specified in the header were written");
+        }
+        recordsWritten += (currSize / RECORD_SIZE);
+        if (0 != currSize % RECORD_SIZE) {
+            recordsWritten++;
         }
         haveUnclosedEntry = false;
     }
 
     /**
-     * Writes bytes to the current tar archive entry. This method
-     * is aware of the current entry and will throw an exception if
-     * you attempt to write bytes past the length specified for the
-     * current entry. The method is also (painfully) aware of the
-     * record buffering required by TarBuffer, and manages buffers
-     * that are not a multiple of recordsize in length, including
-     * assembling records from small buffers.
+     * Writes bytes to the current tar archive entry. This method is aware of the current entry and
+     * will throw an exception if you attempt to write bytes past the length specified for the
+     * current entry.
      *
      * @param wBuf The buffer to write to the archive.
      * @param wOffset The offset in the buffer from which to get bytes.
@@ -380,83 +447,39 @@ public class TarArchiveOutputStream extends ArchiveOutputStream {
         }
         if (currBytes + numToWrite > currSize) {
             throw new IOException("request to write '" + numToWrite
-                                  + "' bytes exceeds size in header of '"
-                                  + currSize + "' bytes for entry '"
-                                  + currName + "'");
-
-            //
-            // We have to deal with assembly!!!
-            // The programmer can be writing little 32 byte chunks for all
-            // we know, and we must assemble complete records for writing.
-            // REVIEW Maybe this should be in TarBuffer? Could that help to
-            // eliminate some of the buffer copying.
-            //
-        }
-
-        if (assemLen > 0) {
-            if (assemLen + numToWrite >= recordBuf.length) {
-                final int aLen = recordBuf.length - assemLen;
-
-                System.arraycopy(assemBuf, 0, recordBuf, 0,
-                                 assemLen);
-                System.arraycopy(wBuf, wOffset, recordBuf,
-                                 assemLen, aLen);
-                writeRecord(recordBuf);
-
-                currBytes += recordBuf.length;
-                wOffset += aLen;
-                numToWrite -= aLen;
-                assemLen = 0;
-            } else {
-                System.arraycopy(wBuf, wOffset, assemBuf, assemLen,
-                                 numToWrite);
-
-                wOffset += numToWrite;
-                assemLen += numToWrite;
-                numToWrite = 0;
-            }
-        }
-
-        //
-        // When we get here we have EITHER:
-        // o An empty "assemble" buffer.
-        // o No bytes to write (numToWrite == 0)
-        //
-        while (numToWrite > 0) {
-            if (numToWrite < recordBuf.length) {
-                System.arraycopy(wBuf, wOffset, assemBuf, assemLen,
-                                 numToWrite);
-
-                assemLen += numToWrite;
-
-                break;
-            }
-
-            writeRecord(wBuf, wOffset);
-
-            final int num = recordBuf.length;
-
-            currBytes += num;
-            numToWrite -= num;
-            wOffset += num;
+                + "' bytes exceeds size in header of '"
+                + currSize + "' bytes for entry '"
+                + currName + "'");
         }
+        out.write(wBuf, wOffset, numToWrite);
+        currBytes += numToWrite;
     }
 
     /**
      * Writes a PAX extended header with the given map as contents.
+     *
      * @since 1.4
      */
     void writePaxHeaders(final TarArchiveEntry entry,
-                         final String entryName,
-                         final Map<String, String> headers) throws IOException {
+        final String entryName,
+        final Map<String, String> headers) throws IOException {
         String name = "./PaxHeaders.X/" + stripTo7Bits(entryName);
         if (name.length() >= TarConstants.NAMELEN) {
             name = name.substring(0, TarConstants.NAMELEN - 1);
         }
         final TarArchiveEntry pex = new TarArchiveEntry(name,
-                                                  TarConstants.LF_PAX_EXTENDED_HEADER_LC);
+            TarConstants.LF_PAX_EXTENDED_HEADER_LC);
         transferModTime(entry, pex);
 
+        final byte[] data = encodeExtendedPaxHeadersContents(headers);
+        pex.setSize(data.length);
+        putArchiveEntry(pex);
+        write(data);
+        closeArchiveEntry();
+    }
+
+    private byte[] encodeExtendedPaxHeadersContents(Map<String, String> headers)
+        throws UnsupportedEncodingException {
         final StringWriter w = new StringWriter();
         for (final Map.Entry<String, String> h : headers.entrySet()) {
             final String key = h.getKey();
@@ -478,11 +501,7 @@ public class TarArchiveOutputStream extends ArchiveOutputStream {
             }
             w.write(line);
         }
-        final byte[] data = w.toString().getBytes(CharsetNames.UTF_8);
-        pex.setSize(data.length);
-        putArchiveEntry(pex);
-        write(data);
-        closeArchiveEntry();
+        return w.toString().getBytes(CharsetNames.UTF_8);
     }
 
     private String stripTo7Bits(final String name) {
@@ -500,8 +519,8 @@ public class TarArchiveOutputStream extends ArchiveOutputStream {
     }
 
     /**
-     * @return true if the character could lead to problems when used
-     * inside a TarArchiveEntry name for a PAX header.
+     * @return true if the character could lead to problems when used inside a TarArchiveEntry name
+     * for a PAX header.
      */
     private boolean shouldBeReplaced(final char c) {
         return c == 0 // would be read as Trailing null
@@ -510,8 +529,8 @@ public class TarArchiveOutputStream extends ArchiveOutputStream {
     }
 
     /**
-     * Write an EOF (end of archive) record to the tar archive.
-     * An EOF record consists of a record of all zeros.
+     * Write an EOF (end of archive) record to the tar archive. An EOF record consists of a record
+     * of all zeros.
      */
     private void writeEOFRecord() throws IOException {
         Arrays.fill(recordBuf, (byte) 0);
@@ -525,13 +544,13 @@ public class TarArchiveOutputStream extends ArchiveOutputStream {
 
     @Override
     public ArchiveEntry createArchiveEntry(final File inputFile, final String entryName)
-            throws IOException {
-        if(finished) {
+        throws IOException {
+        if (finished) {
             throw new IOException("Stream has already been finished");
         }
         return new TarArchiveEntry(inputFile, entryName);
     }
-    
+
     /**
      * Write an archive record to the archive.
      *
@@ -539,38 +558,16 @@ public class TarArchiveOutputStream extends ArchiveOutputStream {
      * @throws IOException on error
      */
     private void writeRecord(final byte[] record) throws IOException {
-        if (record.length != recordSize) {
+        if (record.length != RECORD_SIZE) {
             throw new IOException("record to write has length '"
-                                  + record.length
-                                  + "' which is not the record size of '"
-                                  + recordSize + "'");
+                + record.length
+                + "' which is not the record size of '"
+                + RECORD_SIZE + "'");
         }
 
         out.write(record);
         recordsWritten++;
     }
-    
-    /**
-     * Write an archive record to the archive, where the record may be
-     * inside of a larger array buffer. The buffer must be "offset plus
-     * record size" long.
-     *
-     * @param buf The buffer containing the record data to write.
-     * @param offset The offset of the record data within buf.
-     * @throws IOException on error
-     */
-    private void writeRecord(final byte[] buf, final int offset) throws IOException {
- 
-        if (offset + recordSize > buf.length) {
-            throw new IOException("record has length '" + buf.length
-                                  + "' with offset '" + offset
-                                  + "' which is less than the record size of '"
-                                  + recordSize + "'");
-        }
-
-        out.write(buf, offset, recordSize);
-        recordsWritten++;
-    }
 
     private void padAsNeeded() throws IOException {
         final int start = recordsWritten % recordsPerBlock;
@@ -582,28 +579,28 @@ public class TarArchiveOutputStream extends ArchiveOutputStream {
     }
 
     private void addPaxHeadersForBigNumbers(final Map<String, String> paxHeaders,
-                                            final TarArchiveEntry entry) {
+        final TarArchiveEntry entry) {
         addPaxHeaderForBigNumber(paxHeaders, "size", entry.getSize(),
-                                 TarConstants.MAXSIZE);
+            TarConstants.MAXSIZE);
         addPaxHeaderForBigNumber(paxHeaders, "gid", entry.getLongGroupId(),
-                                 TarConstants.MAXID);
+            TarConstants.MAXID);
         addPaxHeaderForBigNumber(paxHeaders, "mtime",
-                                 entry.getModTime().getTime() / 1000,
-                                 TarConstants.MAXSIZE);
+            entry.getModTime().getTime() / 1000,
+            TarConstants.MAXSIZE);
         addPaxHeaderForBigNumber(paxHeaders, "uid", entry.getLongUserId(),
-                                 TarConstants.MAXID);
+            TarConstants.MAXID);
         // star extensions by J\u00f6rg Schilling
         addPaxHeaderForBigNumber(paxHeaders, "SCHILY.devmajor",
-                                 entry.getDevMajor(), TarConstants.MAXID);
+            entry.getDevMajor(), TarConstants.MAXID);
         addPaxHeaderForBigNumber(paxHeaders, "SCHILY.devminor",
-                                 entry.getDevMinor(), TarConstants.MAXID);
+            entry.getDevMinor(), TarConstants.MAXID);
         // there is no PAX header for file mode
         failForBigNumber("mode", entry.getMode(), TarConstants.MAXID);
     }
 
     private void addPaxHeaderForBigNumber(final Map<String, String> paxHeaders,
-                                          final String header, final long value,
-                                          final long maxValue) {
+        final String header, final long value,
+        final long maxValue) {
         if (value < 0 || value > maxValue) {
             paxHeaders.put(header, String.valueOf(value));
         }
@@ -613,45 +610,43 @@ public class TarArchiveOutputStream extends ArchiveOutputStream {
         failForBigNumber("entry size", entry.getSize(), TarConstants.MAXSIZE);
         failForBigNumberWithPosixMessage("group id", entry.getLongGroupId(), TarConstants.MAXID);
         failForBigNumber("last modification time",
-                         entry.getModTime().getTime() / 1000,
-                         TarConstants.MAXSIZE);
+            entry.getModTime().getTime() / 1000,
+            TarConstants.MAXSIZE);
         failForBigNumber("user id", entry.getLongUserId(), TarConstants.MAXID);
         failForBigNumber("mode", entry.getMode(), TarConstants.MAXID);
         failForBigNumber("major device number", entry.getDevMajor(),
-                         TarConstants.MAXID);
+            TarConstants.MAXID);
         failForBigNumber("minor device number", entry.getDevMinor(),
-                         TarConstants.MAXID);
+            TarConstants.MAXID);
     }
 
     private void failForBigNumber(final String field, final long value, final long maxValue) {
         failForBigNumber(field, value, maxValue, "");
     }
 
-    private void failForBigNumberWithPosixMessage(final String field, final long value, final long maxValue) {
-        failForBigNumber(field, value, maxValue, " Use STAR or POSIX extensions to overcome this limit");
+    private void failForBigNumberWithPosixMessage(final String field, final long value,
+        final long maxValue) {
+        failForBigNumber(field, value, maxValue,
+            " Use STAR or POSIX extensions to overcome this limit");
     }
 
-    private void failForBigNumber(final String field, final long value, final long maxValue, final String additionalMsg) {
+    private void failForBigNumber(final String field, final long value, final long maxValue,
+        final String additionalMsg) {
         if (value < 0 || value > maxValue) {
             throw new RuntimeException(field + " '" + value //NOSONAR
-                    + "' is too big ( > "
-                    + maxValue + " )." + additionalMsg);
+                + "' is too big ( > "
+                + maxValue + " )." + additionalMsg);
         }
     }
 
     /**
      * Handles long file or link names according to the longFileMode setting.
      *
-     * <p>I.e. if the given name is too long to be written to a plain
-     * tar header then
-     * <ul>
-     *   <li>it creates a pax header who's name is given by the
-     *   paxHeaderName parameter if longFileMode is POSIX</li>
-     *   <li>it creates a GNU longlink entry who's type is given by
-     *   the linkType parameter if longFileMode is GNU</li>
-     *   <li>it throws an exception if longFileMode is ERROR</li>
-     *   <li>it truncates the name if longFileMode is TRUNCATE</li>
-     * </ul></p>
+     * <p>I.e. if the given name is too long to be written to a plain tar header then <ul> <li>it
+     * creates a pax header who's name is given by the paxHeaderName parameter if longFileMode is
+     * POSIX</li> <li>it creates a GNU longlink entry who's type is given by the linkType parameter
+     * if longFileMode is GNU</li> <li>it throws an exception if longFileMode is ERROR</li> <li>it
+     * truncates the name if longFileMode is TRUNCATE</li> </ul></p>
      *
      * @param entry entry the name belongs to
      * @param name the name to write
@@ -661,9 +656,9 @@ public class TarArchiveOutputStream extends ArchiveOutputStream {
      * @param fieldName the name of the field
      * @return whether a pax header has been written.
      */
-    private boolean handleLongName(final TarArchiveEntry entry , final String name,
-                                   final Map<String, String> paxHeaders,
-                                   final String paxHeaderName, final byte linkType, final String fieldName)
+    private boolean handleLongName(final TarArchiveEntry entry, final String name,
+        final Map<String, String> paxHeaders,
+        final String paxHeaderName, final byte linkType, final String fieldName)
         throws IOException {
         final ByteBuffer encodedName = zipEncoding.encode(name);
         final int len = encodedName.limit() - encodedName.position();
@@ -675,9 +670,10 @@ public class TarArchiveOutputStream extends ArchiveOutputStream {
             } else if (longFileMode == LONGFILE_GNU) {
                 // create a TarEntry for the LongLink, the contents
                 // of which are the link's name
-                final TarArchiveEntry longLinkEntry = new TarArchiveEntry(TarConstants.GNU_LONGLINK, linkType);
+                final TarArchiveEntry longLinkEntry = new TarArchiveEntry(TarConstants.GNU_LONGLINK,
+                    linkType);
 
-                longLinkEntry.setSize(len + 1l); // +1 for NUL
+                longLinkEntry.setSize(len + 1L); // +1 for NUL
                 transferModTime(entry, longLinkEntry);
                 putArchiveEntry(longLinkEntry);
                 write(encodedName.array(), encodedName.arrayOffset(), len);
@@ -685,8 +681,8 @@ public class TarArchiveOutputStream extends ArchiveOutputStream {
                 closeArchiveEntry();
             } else if (longFileMode != LONGFILE_TRUNCATE) {
                 throw new RuntimeException(fieldName + " '" + name //NOSONAR
-                                           + "' is too long ( > "
-                                           + TarConstants.NAMELEN + " bytes)");
+                    + "' is too long ( > "
+                    + TarConstants.NAMELEN + " bytes)");
             }
         }
         return false;
