@@ -45,13 +45,11 @@ public class TarUtils {
      * ZipEncoding.
      */
     static final ZipEncoding FALLBACK_ENCODING = new ZipEncoding() {
-            @Override
-            public boolean canEncode(final String name) { return true; }
+            public boolean canEncode(String name) { return true; }
 
-            @Override
-            public ByteBuffer encode(final String name) {
+            public ByteBuffer encode(String name) {
                 final int length = name.length();
-                final byte[] buf = new byte[length];
+                byte[] buf = new byte[length];
 
                 // copy until end of input or output is reached.
                 for (int i = 0; i < length; ++i) {
@@ -60,12 +58,12 @@ public class TarUtils {
                 return ByteBuffer.wrap(buf);
             }
 
-            @Override
-            public String decode(final byte[] buffer) {
+            public String decode(byte[] buffer) {
                 final int length = buffer.length;
-                final StringBuilder result = new StringBuilder(length);
+                StringBuilder result = new StringBuilder(length);
 
-                for (final byte b : buffer) {
+                for (int i = 0; i < length; ++i) {
+                    byte b = buffer[i];
                     if (b == 0) { // Trailing null
                         break;
                     }
@@ -123,12 +121,18 @@ public class TarUtils {
             }
         }
 
-        // Trim all trailing NULs and spaces.
-        // The ustar and POSIX tar specs require a trailing NUL or
-        // space but some implementations use the extra digit for big
-        // sizes/uids/gids ...
-        byte trailer = buffer[end - 1];
-        while (start < end && (trailer == 0 || trailer == ' ')) {
+        // Must have trailing NUL or space
+        byte trailer;
+        trailer = buffer[end-1];
+        if (trailer == 0 || trailer == ' '){
+            end--;
+        } else {
+            throw new IllegalArgumentException(
+                    exceptionMessage(buffer, offset, length, end-1, trailer));
+        }
+        // May have additional NULs or spaces
+        trailer = buffer[end - 1];
+        while (start < end - 1 && (trailer == 0 || trailer == ' ')) {
             end--;
             trailer = buffer[end - 1];
         }
@@ -193,7 +197,7 @@ public class TarUtils {
         if (negative) {
             // 2's complement
             val--;
-            val ^= (long) Math.pow(2.0, (length - 1) * 8.0) - 1;
+            val ^= (long) Math.pow(2, (length - 1) * 8) - 1;
         }
         return negative ? -val : val;
     }
@@ -202,7 +206,7 @@ public class TarUtils {
                                               final int offset,
                                               final int length,
                                               final boolean negative) {
-        final byte[] remainder = new byte[length - 1];
+        byte[] remainder = new byte[length - 1];
         System.arraycopy(buffer, offset + 1, remainder, 0, length - 1);
         BigInteger val = new BigInteger(remainder);
         if (negative) {
@@ -233,8 +237,8 @@ public class TarUtils {
     }
 
     // Helper method to generate the exception message
-    private static String exceptionMessage(final byte[] buffer, final int offset,
-            final int length, final int current, final byte currentByte) {
+    private static String exceptionMessage(byte[] buffer, final int offset,
+            final int length, int current, final byte currentByte) {
         // default charset is good enough for an exception message,
         //
         // the alternative was to modify parseOctal and
@@ -259,15 +263,15 @@ public class TarUtils {
      * @param length The maximum number of bytes to parse.
      * @return The entry name.
      */
-    public static String parseName(final byte[] buffer, final int offset, final int length) {
+    public static String parseName(byte[] buffer, final int offset, final int length) {
         try {
             return parseName(buffer, offset, length, DEFAULT_ENCODING);
-        } catch (final IOException ex) {
+        } catch (IOException ex) {
             try {
                 return parseName(buffer, offset, length, FALLBACK_ENCODING);
-            } catch (final IOException ex2) {
+            } catch (IOException ex2) {
                 // impossible
-                throw new RuntimeException(ex2); //NOSONAR
+                throw new RuntimeException(ex2);
             }
         }
     }
@@ -283,9 +287,8 @@ public class TarUtils {
      * @param encoding name of the encoding to use for file names
      * @since 1.4
      * @return The entry name.
-     * @throws IOException on error
      */
-    public static String parseName(final byte[] buffer, final int offset,
+    public static String parseName(byte[] buffer, final int offset,
                                    final int length,
                                    final ZipEncoding encoding)
         throws IOException {
@@ -297,7 +300,7 @@ public class TarUtils {
             }
         }
         if (len > 0) {
-            final byte[] b = new byte[len];
+            byte[] b = new byte[len];
             System.arraycopy(buffer, offset, b, 0, len);
             return encoding.decode(b);
         }
@@ -319,16 +322,16 @@ public class TarUtils {
      * @param length The maximum number of header bytes to copy.
      * @return The updated offset, i.e. offset + length
      */
-    public static int formatNameBytes(final String name, final byte[] buf, final int offset, final int length) {
+    public static int formatNameBytes(String name, byte[] buf, final int offset, final int length) {
         try {
             return formatNameBytes(name, buf, offset, length, DEFAULT_ENCODING);
-        } catch (final IOException ex) {
+        } catch (IOException ex) {
             try {
                 return formatNameBytes(name, buf, offset, length,
                                        FALLBACK_ENCODING);
-            } catch (final IOException ex2) {
+            } catch (IOException ex2) {
                 // impossible
-                throw new RuntimeException(ex2); //NOSONAR
+                throw new RuntimeException(ex2);
             }
         }
     }
@@ -349,9 +352,8 @@ public class TarUtils {
      * @param encoding name of the encoding to use for file names
      * @since 1.4
      * @return The updated offset, i.e. offset + length
-     * @throws IOException on error
      */
-    public static int formatNameBytes(final String name, final byte[] buf, final int offset,
+    public static int formatNameBytes(String name, byte[] buf, final int offset,
                                       final int length,
                                       final ZipEncoding encoding)
         throws IOException {
@@ -380,7 +382,7 @@ public class TarUtils {
      * @param length length of buffer to fill
      * @throws IllegalArgumentException if the value will not fit in the buffer
      */
-    public static void formatUnsignedOctalString(final long value, final byte[] buffer,
+    public static void formatUnsignedOctalString(final long value, byte[] buffer,
             final int offset, final int length) {
         int remaining = length;
         remaining--;
@@ -419,7 +421,7 @@ public class TarUtils {
      * @return The updated offset, i.e offset+length
      * @throws IllegalArgumentException if the value (and trailer) will not fit in the buffer
      */
-    public static int formatOctalBytes(final long value, final byte[] buf, final int offset, final int length) {
+    public static int formatOctalBytes(final long value, byte[] buf, final int offset, final int length) {
 
         int idx=length-2; // For space and trailing null
         formatUnsignedOctalString(value, buf, offset, idx);
@@ -444,9 +446,9 @@ public class TarUtils {
      * @return The updated offset
      * @throws IllegalArgumentException if the value (and trailer) will not fit in the buffer
      */
-    public static int formatLongOctalBytes(final long value, final byte[] buf, final int offset, final int length) {
+    public static int formatLongOctalBytes(final long value, byte[] buf, final int offset, final int length) {
 
-        final int idx=length-1; // For space
+        int idx=length-1; // For space
 
         formatUnsignedOctalString(value, buf, offset, idx);
         buf[offset + idx] = (byte) ' '; // Trailing space
@@ -472,7 +474,7 @@ public class TarUtils {
      * @since 1.4
      */
     public static int formatLongOctalOrBinaryBytes(
-        final long value, final byte[] buf, final int offset, final int length) {
+        final long value, byte[] buf, final int offset, final int length) {
 
         // Check whether we are dealing with UID/GID or SIZE field
         final long maxAsOctalChar = length == TarConstants.UIDLEN ? TarConstants.MAXID : TarConstants.MAXSIZE;
@@ -491,7 +493,7 @@ public class TarUtils {
         return offset + length;
     }
 
-    private static void formatLongBinary(final long value, final byte[] buf,
+    private static void formatLongBinary(final long value, byte[] buf,
                                          final int offset, final int length,
                                          final boolean negative) {
         final int bits = (length - 1) * 8;
@@ -512,11 +514,11 @@ public class TarUtils {
         }
     }
 
-    private static void formatBigIntegerBinary(final long value, final byte[] buf,
+    private static void formatBigIntegerBinary(final long value, byte[] buf,
                                                final int offset,
                                                final int length,
                                                final boolean negative) {
-        final BigInteger val = BigInteger.valueOf(value);
+        BigInteger val = BigInteger.valueOf(value);
         final byte[] b = val.toByteArray();
         final int len = b.length;
         final int off = offset + length - len;
@@ -541,7 +543,7 @@ public class TarUtils {
      * @return The updated value of offset, i.e. offset+length
      * @throws IllegalArgumentException if the value (and trailer) will not fit in the buffer
      */
-    public static int formatCheckSumOctalBytes(final long value, final byte[] buf, final int offset, final int length) {
+    public static int formatCheckSumOctalBytes(final long value, byte[] buf, final int offset, final int length) {
 
         int idx=length-2; // for NUL and space
         formatUnsignedOctalString(value, buf, offset, idx);
@@ -561,7 +563,7 @@ public class TarUtils {
     public static long computeCheckSum(final byte[] buf) {
         long sum = 0;
 
-        for (final byte element : buf) {
+        for (byte element : buf) {
             sum += BYTE_MASK & element;
         }
 
@@ -583,6 +585,16 @@ public class TarUtils {
      * checksum.
      * </blockquote>
      * <p>
+     * In addition there are
+     * <a href="https://issues.apache.org/jira/browse/COMPRESS-117">some tar files</a>
+     * that seem to have parts of their header cleared to zero (no detectable
+     * magic bytes, etc.) but still have a reasonable-looking checksum field
+     * present. It looks like we can detect such cases reasonably well by
+     * checking whether the stored checksum is <em>greater than</em> the
+     * computed unsigned checksum. That check is unlikely to pass on some
+     * random file header, as it would need to have a valid sequence of
+     * octal digits in just the right place.
+     * <p>
      * The return value of this method should be treated as a best-effort
      * heuristic rather than an absolute and final truth. The checksum
      * verification logic may well evolve over time as more special cases
@@ -593,20 +605,28 @@ public class TarUtils {
      * @see <a href="https://issues.apache.org/jira/browse/COMPRESS-191">COMPRESS-191</a>
      * @since 1.5
      */
-    public static boolean verifyCheckSum(final byte[] header) {
-        final long storedSum = parseOctal(header, CHKSUM_OFFSET, CHKSUMLEN);
+    public static boolean verifyCheckSum(byte[] header) {
+        long storedSum = 0;
         long unsignedSum = 0;
         long signedSum = 0;
 
+        int digits = 0;
         for (int i = 0; i < header.length; i++) {
             byte b = header[i];
             if (CHKSUM_OFFSET  <= i && i < CHKSUM_OFFSET + CHKSUMLEN) {
+                if ('0' <= b && b <= '7' && digits++ < 6) {
+                    storedSum = storedSum * 8 + b - '0';
+                } else if (digits > 0) {
+                    digits = 6; // only look at the first octal digit sequence
+                }
                 b = ' ';
             }
             unsignedSum += 0xff & b;
             signedSum += b;
         }
-        return storedSum == unsignedSum || storedSum == signedSum;
+
+        return storedSum == unsignedSum || storedSum == signedSum
+                || storedSum > unsignedSum; // COMPRESS-177
     }
 
 }
