@@ -26,16 +26,21 @@ package org.apache.commons.compress.compressors.bzip2;
 
 import java.io.IOException;
 import java.io.InputStream;
+import java.nio.ByteOrder;
+import java.util.Arrays;
 
 import org.apache.commons.compress.compressors.CompressorInputStream;
+import org.apache.commons.compress.utils.BitInputStream;
+import org.apache.commons.compress.utils.CloseShieldFilterInputStream;
+import org.apache.commons.compress.utils.InputStreamStatistics;
 
 /**
  * An input stream that decompresses from the BZip2 format to be read as any other stream.
- * 
+ *
  * @NotThreadSafe
  */
-public class BZip2CompressorInputStream extends CompressorInputStream implements
-                                                                          BZip2Constants {
+public class BZip2CompressorInputStream extends CompressorInputStream
+    implements BZip2Constants, InputStreamStatistics {
 
     /**
      * Index of the last char in the block, so the block size == last + 1.
@@ -55,13 +60,11 @@ public class BZip2CompressorInputStream extends CompressorInputStream implements
 
     private boolean blockRandomised;
 
-    private int bsBuff;
-    private int bsLive;
     private final CRC crc = new CRC();
 
     private int nInUse;
 
-    private InputStream in;
+    private BitInputStream bin;
     private final boolean decompressConcatenated;
 
     private static final int EOF = 0;
@@ -99,7 +102,7 @@ public class BZip2CompressorInputStream extends CompressorInputStream implements
      * Constructs a new BZip2CompressorInputStream which decompresses bytes
      * read from the specified stream. This doesn't suppprt decompressing
      * concatenated .bz2 files.
-     * 
+     *
      * @param in the InputStream from which this object should be created
      * @throws IOException
      *             if the stream content is malformed or an I/O error occurs.
@@ -125,7 +128,8 @@ public class BZip2CompressorInputStream extends CompressorInputStream implements
      *             if {@code in == null}, the stream content is malformed, or an I/O error occurs.
      */
     public BZip2CompressorInputStream(final InputStream in, final boolean decompressConcatenated) throws IOException {
-        this.in = in;
+        this.bin = new BitInputStream(in == System.in ? new CloseShieldFilterInputStream(in) : in,
+            ByteOrder.BIG_ENDIAN);
         this.decompressConcatenated = decompressConcatenated;
 
         init(true);
@@ -134,7 +138,7 @@ public class BZip2CompressorInputStream extends CompressorInputStream implements
 
     @Override
     public int read() throws IOException {
-        if (this.in != null) {
+        if (this.bin != null) {
             final int r = read0();
             count(r < 0 ? -1 : 1);
             return r;
@@ -144,7 +148,7 @@ public class BZip2CompressorInputStream extends CompressorInputStream implements
 
     /*
      * (non-Javadoc)
-     * 
+     *
      * @see java.io.InputStream#read(byte[], int, int)
      */
     @Override
@@ -160,7 +164,7 @@ public class BZip2CompressorInputStream extends CompressorInputStream implements
             throw new IndexOutOfBoundsException("offs(" + offs + ") + len("
                                                 + len + ") > dest.length(" + dest.length + ").");
         }
-        if (this.in == null) {
+        if (this.bin == null) {
             throw new IOException("stream closed");
         }
         if (len == 0) {
@@ -175,8 +179,15 @@ public class BZip2CompressorInputStream extends CompressorInputStream implements
             count(1);
         }
 
-        final int c = (destOffs == offs) ? -1 : (destOffs - offs);
-        return c;
+        return (destOffs == offs) ? -1 : (destOffs - offs);
+    }
+
+    /**
+     * @since 1.17
+     */
+    @Override
+    public long getCompressedCount() {
+        return bin.getBytesRead();
     }
 
     private void makeMaps() {
@@ -225,17 +236,26 @@ public class BZip2CompressorInputStream extends CompressorInputStream implements
         }
     }
 
+    private int readNextByte(BitInputStream in) throws IOException {
+        long b = in.readBits(8);
+        return (int) b;
+    }
+
     private boolean init(final boolean isFirstStream) throws IOException {
-        if (null == in) {
+        if (null == bin) {
             throw new IOException("No InputStream");
         }
 
-        final int magic0 = this.in.read();
+        if (!isFirstStream) {
+            bin.clearBitCache();
+        }
+
+        final int magic0 = readNextByte(this.bin);
         if (magic0 == -1 && !isFirstStream) {
             return false;
         }
-        final int magic1 = this.in.read();
-        final int magic2 = this.in.read();
+        final int magic1 = readNextByte(this.bin);
+        final int magic2 = readNextByte(this.bin);
 
         if (magic0 != 'B' || magic1 != 'Z' || magic2 != 'h') {
             throw new IOException(isFirstStream
@@ -243,20 +263,20 @@ public class BZip2CompressorInputStream extends CompressorInputStream implements
                     : "Garbage after a valid BZip2 stream");
         }
 
-        final int blockSize = this.in.read();
+        final int blockSize = readNextByte(this.bin);
         if ((blockSize < '1') || (blockSize > '9')) {
             throw new IOException("BZip2 block size is invalid");
         }
 
         this.blockSize100k = blockSize - '0';
 
-        this.bsLive = 0;
         this.computedCombinedCRC = 0;
 
         return true;
     }
 
     private void initBlock() throws IOException {
+        BitInputStream bin = this.bin;
         char magic0;
         char magic1;
         char magic2;
@@ -266,12 +286,12 @@ public class BZip2CompressorInputStream extends CompressorInputStream implements
 
         while (true) {
             // Get the block magic bytes.
-            magic0 = bsGetUByte();
-            magic1 = bsGetUByte();
-            magic2 = bsGetUByte();
-            magic3 = bsGetUByte();
-            magic4 = bsGetUByte();
-            magic5 = bsGetUByte();
+            magic0 = bsGetUByte(bin);
+            magic1 = bsGetUByte(bin);
+            magic2 = bsGetUByte(bin);
+            magic3 = bsGetUByte(bin);
+            magic4 = bsGetUByte(bin);
+            magic5 = bsGetUByte(bin);
 
             // If isn't end of stream magic, break out of the loop.
             if (magic0 != 0x17 || magic1 != 0x72 || magic2 != 0x45
@@ -297,8 +317,8 @@ public class BZip2CompressorInputStream extends CompressorInputStream implements
             this.currentState = EOF;
             throw new IOException("bad block header");
         }
-        this.storedBlockCRC = bsGetInt();
-        this.blockRandomised = bsR(1) == 1;
+        this.storedBlockCRC = bsGetInt(bin);
+        this.blockRandomised = bsR(bin, 1) == 1;
 
         /**
          * Allocate data here instead in constructor, so we do not allocate
@@ -335,7 +355,7 @@ public class BZip2CompressorInputStream extends CompressorInputStream implements
     }
 
     private boolean complete() throws IOException {
-        this.storedCombinedCRC = bsGetInt();
+        this.storedCombinedCRC = bsGetInt(bin);
         this.currentState = EOF;
         this.data = null;
 
@@ -350,53 +370,51 @@ public class BZip2CompressorInputStream extends CompressorInputStream implements
 
     @Override
     public void close() throws IOException {
-        final InputStream inShadow = this.in;
+        final BitInputStream inShadow = this.bin;
         if (inShadow != null) {
             try {
-                if (inShadow != System.in) {
-                    inShadow.close();
-                }
+                inShadow.close();
             } finally {
                 this.data = null;
-                this.in = null;
+                this.bin = null;
             }
         }
     }
 
-    private int bsR(final int n) throws IOException {
-        int bsLiveShadow = this.bsLive;
-        int bsBuffShadow = this.bsBuff;
-
-        if (bsLiveShadow < n) {
-            final InputStream inShadow = this.in;
-            do {
-                final int thech = inShadow.read();
-
-                if (thech < 0) {
-                    throw new IOException("unexpected end of stream");
-                }
-
-                bsBuffShadow = (bsBuffShadow << 8) | thech;
-                bsLiveShadow += 8;
-            } while (bsLiveShadow < n);
-
-            this.bsBuff = bsBuffShadow;
+    /**
+     * read bits from the input stream
+     * @param n the number of bits to read, must not exceed 32?
+     * @return the requested bits combined into an int
+     * @throws IOException
+     */
+    private static int bsR(BitInputStream bin, final int n) throws IOException {
+        long thech = bin.readBits(n);
+        if (thech < 0) {
+            throw new IOException("unexpected end of stream");
         }
-
-        this.bsLive = bsLiveShadow - n;
-        return (bsBuffShadow >> (bsLiveShadow - n)) & ((1 << n) - 1);
+        return (int) thech;
     }
 
-    private boolean bsGetBit() throws IOException {
-        return bsR(1) != 0;
+    private static boolean bsGetBit(BitInputStream bin) throws IOException {
+        return bsR(bin, 1) != 0;
     }
 
-    private char bsGetUByte() throws IOException {
-        return (char) bsR(8);
+    private static char bsGetUByte(BitInputStream bin) throws IOException {
+        return (char) bsR(bin, 8);
     }
 
-    private int bsGetInt() throws IOException {
-        return (((((bsR(8) << 8) | bsR(8)) << 8) | bsR(8)) << 8) | bsR(8);
+    private static int bsGetInt(BitInputStream bin) throws IOException {
+        return bsR(bin, 32);
+    }
+
+    private static void checkBounds(final int checkVal, final int limitExclusive, String name)
+        throws IOException {
+        if (checkVal < 0) {
+            throw new IOException("Corrupted input, " + name + " value negative");
+        }
+        if (checkVal >= limitExclusive) {
+            throw new IOException("Corrupted input, " + name + " value too big");
+        }
     }
 
     /**
@@ -404,7 +422,8 @@ public class BZip2CompressorInputStream extends CompressorInputStream implements
      */
     private static void hbCreateDecodeTables(final int[] limit,
                                              final int[] base, final int[] perm, final char[] length,
-                                             final int minLen, final int maxLen, final int alphaSize) {
+                                             final int minLen, final int maxLen, final int alphaSize)
+        throws IOException {
         for (int i = minLen, pp = 0; i <= maxLen; i++) {
             for (int j = 0; j < alphaSize; j++) {
                 if (length[j] == i) {
@@ -419,7 +438,9 @@ public class BZip2CompressorInputStream extends CompressorInputStream implements
         }
 
         for (int i = 0; i < alphaSize; i++) {
-            base[length[i] + 1]++;
+            final int l = length[i];
+            checkBounds(l, MAX_ALPHA_SIZE, "length");
+            base[l + 1]++;
         }
 
         for (int i = 1, b = base[0]; i < MAX_CODE_LEN; i++) {
@@ -441,6 +462,7 @@ public class BZip2CompressorInputStream extends CompressorInputStream implements
     }
 
     private void recvDecodingTables() throws IOException {
+        final BitInputStream bin = this.bin;
         final Data dataShadow = this.data;
         final boolean[] inUse = dataShadow.inUse;
         final byte[] pos = dataShadow.recvDecodingTables_pos;
@@ -451,20 +473,17 @@ public class BZip2CompressorInputStream extends CompressorInputStream implements
 
         /* Receive the mapping table */
         for (int i = 0; i < 16; i++) {
-            if (bsGetBit()) {
+            if (bsGetBit(bin)) {
                 inUse16 |= 1 << i;
             }
         }
 
-        for (int i = 256; --i >= 0;) {
-            inUse[i] = false;
-        }
-
+        Arrays.fill(inUse, false);
         for (int i = 0; i < 16; i++) {
             if ((inUse16 & (1 << i)) != 0) {
                 final int i16 = i << 4;
                 for (int j = 0; j < 16; j++) {
-                    if (bsGetBit()) {
+                    if (bsGetBit(bin)) {
                         inUse[i16 + j] = true;
                     }
                 }
@@ -473,14 +492,16 @@ public class BZip2CompressorInputStream extends CompressorInputStream implements
 
         makeMaps();
         final int alphaSize = this.nInUse + 2;
-
         /* Now the selectors */
-        final int nGroups = bsR(3);
-        final int nSelectors = bsR(15);
+        final int nGroups = bsR(bin, 3);
+        final int nSelectors = bsR(bin, 15);
+        checkBounds(alphaSize, MAX_ALPHA_SIZE + 1, "alphaSize");
+        checkBounds(nGroups, N_GROUPS + 1, "nGroups");
+        checkBounds(nSelectors, MAX_SELECTORS + 1, "nSelectors");
 
         for (int i = 0; i < nSelectors; i++) {
             int j = 0;
-            while (bsGetBit()) {
+            while (bsGetBit(bin)) {
                 j++;
             }
             selectorMtf[i] = (byte) j;
@@ -493,6 +514,7 @@ public class BZip2CompressorInputStream extends CompressorInputStream implements
 
         for (int i = 0; i < nSelectors; i++) {
             int v = selectorMtf[i] & 0xff;
+            checkBounds(v, N_GROUPS, "selectorMtf");
             final byte tmp = pos[v];
             while (v > 0) {
                 // nearly all times v is zero, 4 in most other cases
@@ -507,11 +529,11 @@ public class BZip2CompressorInputStream extends CompressorInputStream implements
 
         /* Now the coding tables */
         for (int t = 0; t < nGroups; t++) {
-            int curr = bsR(5);
+            int curr = bsR(bin, 5);
             final char[] len_t = len[t];
             for (int i = 0; i < alphaSize; i++) {
-                while (bsGetBit()) {
-                    curr += bsGetBit() ? -1 : 1;
+                while (bsGetBit(bin)) {
+                    curr += bsGetBit(bin) ? -1 : 1;
                 }
                 len_t[i] = (char) curr;
             }
@@ -525,7 +547,7 @@ public class BZip2CompressorInputStream extends CompressorInputStream implements
      * Called by recvDecodingTables() exclusively.
      */
     private void createHuffmanDecodingTables(final int alphaSize,
-                                             final int nGroups) {
+                                             final int nGroups) throws IOException {
         final Data dataShadow = this.data;
         final char[][] len = dataShadow.temp_charArray2d;
         final int[] minLens = dataShadow.minLens;
@@ -553,10 +575,10 @@ public class BZip2CompressorInputStream extends CompressorInputStream implements
     }
 
     private void getAndMoveToFrontDecode() throws IOException {
-        this.origPtr = bsR(24);
+        final BitInputStream bin = this.bin;
+        this.origPtr = bsR(bin, 24);
         recvDecodingTables();
 
-        final InputStream inShadow = this.in;
         final Data dataShadow = this.data;
         final byte[] ll8 = dataShadow.ll8;
         final int[] unzftab = dataShadow.unzftab;
@@ -582,11 +604,10 @@ public class BZip2CompressorInputStream extends CompressorInputStream implements
         int groupNo = 0;
         int groupPos = G_SIZE - 1;
         final int eob = this.nInUse + 1;
-        int nextSym = getAndMoveToFrontDecode0(0);
-        int bsBuffShadow = this.bsBuff;
-        int bsLiveShadow = this.bsLive;
+        int nextSym = getAndMoveToFrontDecode0();
         int lastShadow = -1;
         int zt = selector[groupNo] & 0xff;
+        checkBounds(zt, N_GROUPS, "zt");
         int[] base_zt = base[zt];
         int[] limit_zt = limit[zt];
         int[] perm_zt = perm[zt];
@@ -607,7 +628,9 @@ public class BZip2CompressorInputStream extends CompressorInputStream implements
 
                     if (groupPos == 0) {
                         groupPos = G_SIZE - 1;
-                        zt = selector[++groupNo] & 0xff;
+                        checkBounds(++groupNo, MAX_SELECTORS, "groupNo");
+                        zt = selector[groupNo] & 0xff;
+                        checkBounds(zt, N_GROUPS, "zt");
                         base_zt = base[zt];
                         limit_zt = limit[zt];
                         perm_zt = perm[zt];
@@ -617,57 +640,39 @@ public class BZip2CompressorInputStream extends CompressorInputStream implements
                     }
 
                     int zn = minLens_zt;
-
-                    // Inlined:
-                    // int zvec = bsR(zn);
-                    while (bsLiveShadow < zn) {
-                        final int thech = inShadow.read();
-                        if (thech >= 0) {
-                            bsBuffShadow = (bsBuffShadow << 8) | thech;
-                            bsLiveShadow += 8;
-                            continue;
-                        }
-                        throw new IOException("unexpected end of stream");
+                    checkBounds(zn, MAX_ALPHA_SIZE, "zn");
+                    int zvec = bsR(bin, zn);
+                    while(zvec > limit_zt[zn]) {
+                        checkBounds(++zn, MAX_ALPHA_SIZE, "zn");
+                        zvec = (zvec << 1) | bsR(bin, 1);
                     }
-                    int zvec = (bsBuffShadow >> (bsLiveShadow - zn))
-                        & ((1 << zn) - 1);
-                    bsLiveShadow -= zn;
-
-                    while (zvec > limit_zt[zn]) {
-                        zn++;
-                        while (bsLiveShadow < 1) {
-                            final int thech = inShadow.read();
-                            if (thech >= 0) {
-                                bsBuffShadow = (bsBuffShadow << 8) | thech;
-                                bsLiveShadow += 8;
-                                continue;
-                            }
-                            throw new IOException(
-                                                  "unexpected end of stream");
-                        }
-                        bsLiveShadow--;
-                        zvec = (zvec << 1)
-                            | ((bsBuffShadow >> bsLiveShadow) & 1);
-                    }
-                    nextSym = perm_zt[zvec - base_zt[zn]];
+                    final int tmp = zvec - base_zt[zn];
+                    checkBounds(tmp, MAX_ALPHA_SIZE, "zvec");
+                    nextSym = perm_zt[tmp];
                 }
 
-                final byte ch = seqToUnseq[yy[0]];
+                final int yy0 = yy[0];
+                checkBounds(yy0, 256, "yy");
+                final byte ch = seqToUnseq[yy0];
                 unzftab[ch & 0xff] += s + 1;
 
-                while (s-- >= 0) {
-                    ll8[++lastShadow] = ch;
-                }
+                final int from = ++lastShadow;
+                lastShadow += s;
+                Arrays.fill(ll8, from, lastShadow + 1, ch);
 
                 if (lastShadow >= limitLast) {
-                    throw new IOException("block overrun");
+                    throw new IOException("block overrun while expanding RLE in MTF, "
+                        + lastShadow + " exceeds " + limitLast);
                 }
             } else {
                 if (++lastShadow >= limitLast) {
-                    throw new IOException("block overrun");
+                    throw new IOException("block overrun in MTF, "
+                        + lastShadow + " exceeds " + limitLast);
                 }
+                checkBounds(nextSym, 256 + 1, "nextSym");
 
                 final char tmp = yy[nextSym - 1];
+                checkBounds(tmp, 256, "yy");
                 unzftab[seqToUnseq[tmp] & 0xff]++;
                 ll8[lastShadow] = seqToUnseq[tmp];
 
@@ -688,7 +693,9 @@ public class BZip2CompressorInputStream extends CompressorInputStream implements
 
                 if (groupPos == 0) {
                     groupPos = G_SIZE - 1;
-                    zt = selector[++groupNo] & 0xff;
+                    checkBounds(++groupNo, MAX_SELECTORS, "groupNo");
+                    zt = selector[groupNo] & 0xff;
+                    checkBounds(zt, N_GROUPS, "zt");
                     base_zt = base[zt];
                     limit_zt = limit[zt];
                     perm_zt = perm[zt];
@@ -698,75 +705,37 @@ public class BZip2CompressorInputStream extends CompressorInputStream implements
                 }
 
                 int zn = minLens_zt;
-
-                // Inlined:
-                // int zvec = bsR(zn);
-                while (bsLiveShadow < zn) {
-                    final int thech = inShadow.read();
-                    if (thech >= 0) {
-                        bsBuffShadow = (bsBuffShadow << 8) | thech;
-                        bsLiveShadow += 8;
-                        continue;
-                    }
-                    throw new IOException("unexpected end of stream");
+                checkBounds(zn, MAX_ALPHA_SIZE, "zn");
+                int zvec = bsR(bin, zn);
+                while(zvec > limit_zt[zn]) {
+                    checkBounds(++zn, MAX_ALPHA_SIZE, "zn");
+                    zvec = (zvec << 1) | bsR(bin, 1);
                 }
-                int zvec = (bsBuffShadow >> (bsLiveShadow - zn))
-                    & ((1 << zn) - 1);
-                bsLiveShadow -= zn;
-
-                while (zvec > limit_zt[zn]) {
-                    zn++;
-                    while (bsLiveShadow < 1) {
-                        final int thech = inShadow.read();
-                        if (thech >= 0) {
-                            bsBuffShadow = (bsBuffShadow << 8) | thech;
-                            bsLiveShadow += 8;
-                            continue;
-                        }
-                        throw new IOException("unexpected end of stream");
-                    }
-                    bsLiveShadow--;
-                    zvec = (zvec << 1) | ((bsBuffShadow >> bsLiveShadow) & 1);
-                }
-                nextSym = perm_zt[zvec - base_zt[zn]];
+                final int idx = zvec - base_zt[zn];
+                checkBounds(idx, MAX_ALPHA_SIZE, "zvec");
+                nextSym = perm_zt[idx];
             }
         }
 
         this.last = lastShadow;
-        this.bsLive = bsLiveShadow;
-        this.bsBuff = bsBuffShadow;
     }
 
-    private int getAndMoveToFrontDecode0(final int groupNo) throws IOException {
-        final InputStream inShadow = this.in;
+    private int getAndMoveToFrontDecode0() throws IOException {
         final Data dataShadow = this.data;
-        final int zt = dataShadow.selector[groupNo] & 0xff;
+        final int zt = dataShadow.selector[0] & 0xff;
+        checkBounds(zt, N_GROUPS, "zt");
         final int[] limit_zt = dataShadow.limit[zt];
         int zn = dataShadow.minLens[zt];
-        int zvec = bsR(zn);
-        int bsLiveShadow = this.bsLive;
-        int bsBuffShadow = this.bsBuff;
-
+        checkBounds(zn, MAX_ALPHA_SIZE, "zn");
+        int zvec = bsR(bin, zn);
         while (zvec > limit_zt[zn]) {
-            zn++;
-            while (bsLiveShadow < 1) {
-                final int thech = inShadow.read();
-
-                if (thech >= 0) {
-                    bsBuffShadow = (bsBuffShadow << 8) | thech;
-                    bsLiveShadow += 8;
-                    continue;
-                }
-                throw new IOException("unexpected end of stream");
-            }
-            bsLiveShadow--;
-            zvec = (zvec << 1) | ((bsBuffShadow >> bsLiveShadow) & 1);
+            checkBounds(++zn, MAX_ALPHA_SIZE, "zn");
+            zvec = (zvec << 1) | bsR(bin, 1);
         }
+        final int tmp = zvec - dataShadow.base[zt][zn];
+        checkBounds(tmp, MAX_ALPHA_SIZE, "zvec");
 
-        this.bsLive = bsLiveShadow;
-        this.bsBuff = bsBuffShadow;
-
-        return dataShadow.perm[zt][zvec - dataShadow.base[zt][zn]];
+        return dataShadow.perm[zt][tmp];
     }
 
     private int setupBlock() throws IOException {
@@ -775,7 +744,8 @@ public class BZip2CompressorInputStream extends CompressorInputStream implements
         }
 
         final int[] cftab = this.data.cftab;
-        final int[] tt = this.data.initTT(this.last + 1);
+        final int ttLen = this.last + 1;
+        final int[] tt = this.data.initTT(ttLen);
         final byte[] ll8 = this.data.ll8;
         cftab[0] = 0;
         System.arraycopy(this.data.unzftab, 0, cftab, 1, 256);
@@ -786,7 +756,9 @@ public class BZip2CompressorInputStream extends CompressorInputStream implements
         }
 
         for (int i = 0, lastShadow = this.last; i <= lastShadow; i++) {
-            tt[cftab[ll8[i] & 0xff]++] = i;
+            final int tmp = cftab[ll8[i] & 0xff]++;
+            checkBounds(tmp, ttLen, "tt index");
+            tt[tmp] = i;
         }
 
         if ((this.origPtr < 0) || (this.origPtr >= tt.length)) {
@@ -810,6 +782,7 @@ public class BZip2CompressorInputStream extends CompressorInputStream implements
         if (this.su_i2 <= this.last) {
             this.su_chPrev = this.su_ch2;
             int su_ch2Shadow = this.data.ll8[this.su_tPos] & 0xff;
+            checkBounds(this.su_tPos, this.data.tt.length, "su_tPos");
             this.su_tPos = this.data.tt[this.su_tPos];
             if (this.su_rNToGo == 0) {
                 this.su_rNToGo = Rand.rNums(this.su_rTPos) - 1;
@@ -835,6 +808,7 @@ public class BZip2CompressorInputStream extends CompressorInputStream implements
             this.su_chPrev = this.su_ch2;
             final int su_ch2Shadow = this.data.ll8[this.su_tPos] & 0xff;
             this.su_ch2 = su_ch2Shadow;
+            checkBounds(this.su_tPos, this.data.tt.length, "su_tPos");
             this.su_tPos = this.data.tt[this.su_tPos];
             this.su_i2++;
             this.currentState = NO_RAND_PART_B_STATE;
@@ -854,6 +828,7 @@ public class BZip2CompressorInputStream extends CompressorInputStream implements
             return setupRandPartA();
         } else if (++this.su_count >= 4) {
             this.su_z = (char) (this.data.ll8[this.su_tPos] & 0xff);
+            checkBounds(this.su_tPos, this.data.tt.length, "su_tPos");
             this.su_tPos = this.data.tt[this.su_tPos];
             if (this.su_rNToGo == 0) {
                 this.su_rNToGo = Rand.rNums(this.su_rTPos) - 1;
@@ -892,6 +867,7 @@ public class BZip2CompressorInputStream extends CompressorInputStream implements
             this.su_count = 1;
             return setupNoRandPartA();
         } else if (++this.su_count >= 4) {
+            checkBounds(this.su_tPos, this.data.ll8.length, "su_tPos");
             this.su_z = (char) (this.data.ll8[this.su_tPos] & 0xff);
             this.su_tPos = this.data.tt[this.su_tPos];
             this.su_j2 = 0;
@@ -955,7 +931,7 @@ public class BZip2CompressorInputStream extends CompressorInputStream implements
 
         /**
          * Initializes the {@link #tt} array.
-         * 
+         *
          * This method is called when the required length of the array is known.
          * I don't initialize it at construction time to avoid unneccessary
          * memory allocation when compressing small files.
@@ -978,33 +954,17 @@ public class BZip2CompressorInputStream extends CompressorInputStream implements
 
     /**
      * Checks if the signature matches what is expected for a bzip2 file.
-     * 
+     *
      * @param signature
      *            the bytes to check
      * @param length
      *            the number of bytes to check
      * @return true, if this stream is a bzip2 compressed stream, false otherwise
-     * 
+     *
      * @since 1.1
      */
     public static boolean matches(final byte[] signature, final int length) {
-
-        if (length < 3) {
-            return false;
-        }
-
-        if (signature[0] != 'B') {
-            return false;
-        }
-
-        if (signature[1] != 'Z') {
-            return false;
-        }
-
-        if (signature[2] != 'h') {
-            return false;
-        }
-
-        return true;
+        return length >= 3 && signature[0] == 'B' &&
+                signature[1] == 'Z' && signature[2] == 'h';
     }
 }
