@@ -13,7 +13,7 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- * 
+ *
  */
 
 package org.apache.commons.compress;
@@ -25,6 +25,7 @@ import java.io.BufferedReader;
 import java.io.File;
 import java.io.FileReader;
 import java.io.FilenameFilter;
+import java.net.URISyntaxException;
 import java.util.ArrayList;
 import java.util.Collection;
 
@@ -38,9 +39,9 @@ import org.junit.runners.Parameterized.Parameters;
 
 /**
  * Test that can read various archive file examples.
- * 
+ *
  * This is a very simple implementation.
- * 
+ *
  * Files must be in resources/archives, and there must be a file.txt containing
  * the list of files in the archives.
  */
@@ -48,9 +49,17 @@ import org.junit.runners.Parameterized.Parameters;
 public class ArchiveReadTest extends AbstractTestCase {
 
     private static final ClassLoader CLASSLOADER = ArchiveReadTest.class.getClassLoader();
-    private static final File ARCDIR = new File(CLASSLOADER.getResource("archives").getFile());
+    private static final File ARCDIR;
     private static final ArrayList<String> FILELIST = new ArrayList<>();
 
+    static {
+        try {
+            ARCDIR = new File(CLASSLOADER.getResource("archives").toURI());
+        } catch (URISyntaxException e) {
+            throw new RuntimeException(e);
+        }
+    }
+
     private final File file;
 
     public ArchiveReadTest(final String file){
@@ -81,7 +90,7 @@ public class ArchiveReadTest extends AbstractTestCase {
             public boolean accept(final File dir, final String name) {
                 return !name.endsWith(".txt");
             }
-        })) 
+        }))
         {
             params.add(new Object[] { f });
         }
