@@ -22,14 +22,17 @@ import static org.junit.Assert.*;
 
 import java.io.ByteArrayInputStream;
 import java.io.File;
-import java.io.FileInputStream; 	
-import java.io.FileOutputStream; 	
-import java.io.IOException; 	
-import java.io.InputStream; 	
+import java.io.FileInputStream;
+import java.io.FileOutputStream;
+import java.io.IOException;
+import java.io.InputStream;
 import java.io.OutputStream;
 import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.Enumeration;
+import java.util.HashMap;
 import java.util.List;
+import java.util.Map;
 import java.util.zip.ZipEntry;
 import java.util.zip.ZipException;
 
@@ -37,11 +40,12 @@ import org.apache.commons.compress.AbstractTestCase;
 import org.apache.commons.compress.archivers.zip.Zip64Mode;
 import org.apache.commons.compress.archivers.zip.ZipArchiveEntry;
 import org.apache.commons.compress.archivers.zip.ZipArchiveEntryPredicate;
-import org.apache.commons.compress.archivers.zip.ZipArchiveInputStream; 	
-import org.apache.commons.compress.archivers.zip.ZipArchiveOutputStream; 	
-import org.apache.commons.compress.archivers.zip.ZipFile; 	
+import org.apache.commons.compress.archivers.zip.ZipArchiveInputStream;
+import org.apache.commons.compress.archivers.zip.ZipArchiveOutputStream;
+import org.apache.commons.compress.archivers.zip.ZipFile;
 import org.apache.commons.compress.archivers.zip.ZipMethod;
 import org.apache.commons.compress.utils.IOUtils;
+import org.apache.commons.compress.utils.InputStreamStatistics;
 import org.apache.commons.compress.utils.SeekableInMemoryByteChannel;
 import org.junit.Assert;
 import org.junit.Test;
@@ -124,8 +128,8 @@ public final class ZipTestCase extends AbstractTestCase {
         IOUtils.readFully(new FileInputStream(file1), file1Contents);
         IOUtils.readFully(new FileInputStream(file2), file2Contents);
 
-        SeekableInMemoryByteChannel c = new SeekableInMemoryByteChannel();
-        try (ZipArchiveOutputStream os = new ZipArchiveOutputStream(c)) {
+        SeekableInMemoryByteChannel channel = new SeekableInMemoryByteChannel();
+        try (ZipArchiveOutputStream os = new ZipArchiveOutputStream(channel)) {
             os.putArchiveEntry(new ZipArchiveEntry("testdata/test1.xml"));
             os.write(file1Contents);
             os.closeArchiveEntry();
@@ -139,7 +143,7 @@ public final class ZipTestCase extends AbstractTestCase {
         final List<byte[]> results = new ArrayList<>();
 
         try (ArchiveInputStream in = new ArchiveStreamFactory()
-             .createArchiveInputStream("zip", new ByteArrayInputStream(c.array()))) {
+             .createArchiveInputStream("zip", new ByteArrayInputStream(channel.array()))) {
 
             ZipArchiveEntry entry;
             while((entry = (ZipArchiveEntry)in.getNextEntry()) != null) {
@@ -160,17 +164,17 @@ public final class ZipTestCase extends AbstractTestCase {
     @Test
     public void testZipUnarchive() throws Exception {
         final File input = getFile("bla.zip");
-        final InputStream is = new FileInputStream(input);
-        final ArchiveInputStream in = new ArchiveStreamFactory().createArchiveInputStream("zip", is);
-        final ZipArchiveEntry entry = (ZipArchiveEntry)in.getNextEntry();
-        final OutputStream out = new FileOutputStream(new File(dir, entry.getName()));
-        IOUtils.copy(in, out);
-        out.close();
-        in.close();
+        try (final InputStream is = new FileInputStream(input);
+                final ArchiveInputStream in = new ArchiveStreamFactory().createArchiveInputStream("zip", is)) {
+            final ZipArchiveEntry entry = (ZipArchiveEntry) in.getNextEntry();
+            try (final OutputStream out = new FileOutputStream(new File(dir, entry.getName()))) {
+                IOUtils.copy(in, out);
+            }
+        }
     }
 
     /**
-     * Test case for 
+     * Test case for
      * <a href="https://issues.apache.org/jira/browse/COMPRESS-208"
      * >COMPRESS-208</a>.
      */
@@ -197,7 +201,7 @@ public final class ZipTestCase extends AbstractTestCase {
         assertTrue(bla.canReadEntryData(bla.getEntry("test1.xml")));
         bla.close();
         */
-        
+
         final ZipFile moby = new ZipFile(getFile("moby.zip"));
         final ZipArchiveEntry entry = moby.getEntry("README");
         assertEquals("method", ZipMethod.TOKENIZATION.getCode(), entry.getMethod());
@@ -206,7 +210,7 @@ public final class ZipTestCase extends AbstractTestCase {
     }
 
     /**
-     * Test case for being able to skip an entry in an 
+     * Test case for being able to skip an entry in an
      * {@link ZipArchiveInputStream} even if the compression method of that
      * entry is unsupported.
      *
@@ -234,9 +238,9 @@ public final class ZipTestCase extends AbstractTestCase {
      * Checks if all entries from a nested archive can be read.
      * The archive: OSX_ArchiveWithNestedArchive.zip contains:
      * NestedArchiv.zip and test.xml3.
-     * 
+     *
      * The nested archive:  NestedArchive.zip contains test1.xml and test2.xml
-     * 
+     *
      * @throws Exception
      */
     @Test
@@ -369,28 +373,28 @@ public final class ZipTestCase extends AbstractTestCase {
         final File reference = createReferenceFile(tmp[0], Zip64Mode.Never, "expected.");
 
         final File a1 = File.createTempFile("src1.", ".zip", tmp[0]);
-        final ZipArchiveOutputStream zos = new ZipArchiveOutputStream(a1);
-        zos.setUseZip64(Zip64Mode.Never);
-        createFirstEntry(zos).close();
+        try (final ZipArchiveOutputStream zos = new ZipArchiveOutputStream(a1)) {
+            zos.setUseZip64(Zip64Mode.Never);
+            createFirstEntry(zos).close();
+        }
 
         final File a2 = File.createTempFile("src2.", ".zip", tmp[0]);
-        final ZipArchiveOutputStream zos1 = new ZipArchiveOutputStream(a2);
-        zos1.setUseZip64(Zip64Mode.Never);
-        createSecondEntry(zos1).close();
+        try (final ZipArchiveOutputStream zos1 = new ZipArchiveOutputStream(a2)) {
+            zos1.setUseZip64(Zip64Mode.Never);
+            createSecondEntry(zos1).close();
+        }
 
-        final ZipFile zf1 = new ZipFile(a1);
-        final ZipFile zf2 = new ZipFile(a2);
-        final File fileResult = File.createTempFile("file-actual.", ".zip", tmp[0]);
-        final ZipArchiveOutputStream zos2 = new ZipArchiveOutputStream(fileResult);
-        zf1.copyRawEntries(zos2, allFilesPredicate);
-        zf2.copyRawEntries(zos2, allFilesPredicate);
-        zos2.close();
-        // copyRawEntries does not add superfluous zip64 header like regular zip output stream
-        // does when using Zip64Mode.AsNeeded so all the source material has to be Zip64Mode.Never,
-        // if exact binary equality is to be achieved
-        assertSameFileContents(reference, fileResult);
-        zf1.close();
-        zf2.close();
+        try (final ZipFile zf1 = new ZipFile(a1); final ZipFile zf2 = new ZipFile(a2)) {
+            final File fileResult = File.createTempFile("file-actual.", ".zip", tmp[0]);
+            try (final ZipArchiveOutputStream zos2 = new ZipArchiveOutputStream(fileResult)) {
+                zf1.copyRawEntries(zos2, allFilesPredicate);
+                zf2.copyRawEntries(zos2, allFilesPredicate);
+            }
+            // copyRawEntries does not add superfluous zip64 header like regular zip output stream
+            // does when using Zip64Mode.AsNeeded so all the source material has to be Zip64Mode.Never,
+            // if exact binary equality is to be achieved
+            assertSameFileContents(reference, fileResult);
+        }
     }
 
     @Test
@@ -399,24 +403,25 @@ public final class ZipTestCase extends AbstractTestCase {
 
         final File[] tmp = createTempDirAndFile();
         final File reference = File.createTempFile("z64reference.", ".zip", tmp[0]);
-        final ZipArchiveOutputStream zos1 = new ZipArchiveOutputStream(reference);
-        zos1.setUseZip64(Zip64Mode.Always);
-        createFirstEntry(zos1);
-        zos1.close();
+        try (final ZipArchiveOutputStream zos1 = new ZipArchiveOutputStream(reference)) {
+            zos1.setUseZip64(Zip64Mode.Always);
+            createFirstEntry(zos1);
+        }
 
         final File a1 = File.createTempFile("zip64src.", ".zip", tmp[0]);
-        final ZipArchiveOutputStream zos = new ZipArchiveOutputStream(a1);
-        zos.setUseZip64(Zip64Mode.Always);
-        createFirstEntry(zos).close();
+        try (final ZipArchiveOutputStream zos = new ZipArchiveOutputStream(a1)) {
+            zos.setUseZip64(Zip64Mode.Always);
+            createFirstEntry(zos).close();
+        }
 
-        final ZipFile zf1 = new ZipFile(a1);
         final File fileResult = File.createTempFile("file-actual.", ".zip", tmp[0]);
-        final ZipArchiveOutputStream zos2 = new ZipArchiveOutputStream(fileResult);
-        zos2.setUseZip64(Zip64Mode.Always);
-        zf1.copyRawEntries(zos2, allFilesPredicate);
-        zos2.close();
-        assertSameFileContents(reference, fileResult);
-        zf1.close();
+        try (final ZipFile zf1 = new ZipFile(a1)) {
+            try (final ZipArchiveOutputStream zos2 = new ZipArchiveOutputStream(fileResult)) {
+                zos2.setUseZip64(Zip64Mode.Always);
+                zf1.copyRawEntries(zos2, allFilesPredicate);
+            }
+            assertSameFileContents(reference, fileResult);
+        }
     }
 
     @Test
@@ -425,27 +430,28 @@ public final class ZipTestCase extends AbstractTestCase {
         final File[] tmp = createTempDirAndFile();
 
         final File a1 = File.createTempFile("unixModeBits.", ".zip", tmp[0]);
-        final ZipArchiveOutputStream zos = new ZipArchiveOutputStream(a1);
+        try (final ZipArchiveOutputStream zos = new ZipArchiveOutputStream(a1)) {
 
-        final ZipArchiveEntry archiveEntry = new ZipArchiveEntry("fred");
-        archiveEntry.setUnixMode(0664);
-        archiveEntry.setMethod(ZipEntry.DEFLATED);
-        zos.addRawArchiveEntry(archiveEntry, new ByteArrayInputStream("fud".getBytes()));
-        zos.close();
+            final ZipArchiveEntry archiveEntry = new ZipArchiveEntry("fred");
+            archiveEntry.setUnixMode(0664);
+            archiveEntry.setMethod(ZipEntry.DEFLATED);
+            zos.addRawArchiveEntry(archiveEntry, new ByteArrayInputStream("fud".getBytes()));
+        }
 
-        final ZipFile zf1 = new ZipFile(a1);
-        final ZipArchiveEntry fred = zf1.getEntry("fred");
-        assertEquals(0664, fred.getUnixMode());
-        zf1.close();
+        try (final ZipFile zf1 = new ZipFile(a1)) {
+            final ZipArchiveEntry fred = zf1.getEntry("fred");
+            assertEquals(0664, fred.getUnixMode());
+        }
     }
 
-    private File createReferenceFile(final File directory, final Zip64Mode zipMode, final String prefix) throws IOException {
+    private File createReferenceFile(final File directory, final Zip64Mode zipMode, final String prefix)
+            throws IOException {
         final File reference = File.createTempFile(prefix, ".zip", directory);
-        final ZipArchiveOutputStream zos = new ZipArchiveOutputStream(reference);
-        zos.setUseZip64(zipMode);
-        createFirstEntry(zos);
-        createSecondEntry(zos);
-        zos.close();
+        try (final ZipArchiveOutputStream zos = new ZipArchiveOutputStream(reference)) {
+            zos.setUseZip64(zipMode);
+            createFirstEntry(zos);
+            createSecondEntry(zos);
+        }
         return reference;
     }
 
@@ -462,39 +468,37 @@ public final class ZipTestCase extends AbstractTestCase {
 
     private void assertSameFileContents(final File expectedFile, final File actualFile) throws IOException {
         final int size = (int) Math.max(expectedFile.length(), actualFile.length());
-        final ZipFile expected = new ZipFile(expectedFile);
-        final ZipFile actual = new ZipFile(actualFile);
-        final byte[] expectedBuf = new byte[size];
-        final byte[] actualBuf = new byte[size];
+        try (final ZipFile expected = new ZipFile(expectedFile); final ZipFile actual = new ZipFile(actualFile)) {
+            final byte[] expectedBuf = new byte[size];
+            final byte[] actualBuf = new byte[size];
 
-        final Enumeration<ZipArchiveEntry> actualInOrder = actual.getEntriesInPhysicalOrder();
-        final Enumeration<ZipArchiveEntry> expectedInOrder = expected.getEntriesInPhysicalOrder();
+            final Enumeration<ZipArchiveEntry> actualInOrder = actual.getEntriesInPhysicalOrder();
+            final Enumeration<ZipArchiveEntry> expectedInOrder = expected.getEntriesInPhysicalOrder();
 
-        while (actualInOrder.hasMoreElements()){
-            final ZipArchiveEntry actualElement = actualInOrder.nextElement();
-            final ZipArchiveEntry expectedElement = expectedInOrder.nextElement();
-            assertEquals( expectedElement.getName(), actualElement.getName());
-            // Don't compare timestamps since they may vary;
-            // there's no support for stubbed out clock (TimeSource) in ZipArchiveOutputStream
-            assertEquals( expectedElement.getMethod(), actualElement.getMethod());
-            assertEquals( expectedElement.getGeneralPurposeBit(), actualElement.getGeneralPurposeBit());
-            assertEquals( expectedElement.getCrc(), actualElement.getCrc());
-            assertEquals( expectedElement.getCompressedSize(), actualElement.getCompressedSize());
-            assertEquals( expectedElement.getSize(), actualElement.getSize());
-            assertEquals( expectedElement.getExternalAttributes(), actualElement.getExternalAttributes());
-            assertEquals( expectedElement.getInternalAttributes(), actualElement.getInternalAttributes());
+            while (actualInOrder.hasMoreElements()) {
+                final ZipArchiveEntry actualElement = actualInOrder.nextElement();
+                final ZipArchiveEntry expectedElement = expectedInOrder.nextElement();
+                assertEquals(expectedElement.getName(), actualElement.getName());
+                // Don't compare timestamps since they may vary;
+                // there's no support for stubbed out clock (TimeSource) in ZipArchiveOutputStream
+                assertEquals(expectedElement.getMethod(), actualElement.getMethod());
+                assertEquals(expectedElement.getGeneralPurposeBit(), actualElement.getGeneralPurposeBit());
+                assertEquals(expectedElement.getCrc(), actualElement.getCrc());
+                assertEquals(expectedElement.getCompressedSize(), actualElement.getCompressedSize());
+                assertEquals(expectedElement.getSize(), actualElement.getSize());
+                assertEquals(expectedElement.getExternalAttributes(), actualElement.getExternalAttributes());
+                assertEquals(expectedElement.getInternalAttributes(), actualElement.getInternalAttributes());
+
+                final InputStream actualIs = actual.getInputStream(actualElement);
+                final InputStream expectedIs = expected.getInputStream(expectedElement);
+                IOUtils.readFully(expectedIs, expectedBuf);
+                IOUtils.readFully(actualIs, actualBuf);
+                expectedIs.close();
+                actualIs.close();
+                Assert.assertArrayEquals(expectedBuf, actualBuf); // Buffers are larger than payload. dont care
+            }
 
-            final InputStream actualIs = actual.getInputStream(actualElement);
-            final InputStream expectedIs = expected.getInputStream(expectedElement);
-            IOUtils.readFully(expectedIs, expectedBuf);
-            IOUtils.readFully(actualIs, actualBuf);
-            expectedIs.close();
-            actualIs.close();
-            Assert.assertArrayEquals(expectedBuf, actualBuf); // Buffers are larger than payload. dont care
         }
-
-        expected.close();
-        actual.close();
     }
 
 
@@ -598,4 +602,109 @@ public final class ZipTestCase extends AbstractTestCase {
             rmdir(tmp[0]);
         }
     }
+
+    @Test
+    public void inputStreamStatisticsOfZipBombExcel() throws IOException, ArchiveException {
+        Map<String, List<Long>> expected = new HashMap<String, List<Long>>() {{
+            put("[Content_Types].xml", Arrays.asList(8390036L, 8600L));
+            put("xl/worksheets/sheet1.xml", Arrays.asList(1348L, 508L));
+        }};
+        testInputStreamStatistics("zipbomb.xlsx", expected);
+    }
+
+    @Test
+    public void inputStreamStatisticsForImplodedEntry() throws IOException, ArchiveException {
+        Map<String, List<Long>> expected = new HashMap<String, List<Long>>() {{
+            put("LICENSE.TXT", Arrays.asList(11560L, 4131L));
+        }};
+        testInputStreamStatistics("imploding-8Kdict-3trees.zip", expected);
+    }
+
+    @Test
+    public void inputStreamStatisticsForShrunkEntry() throws IOException, ArchiveException {
+        Map<String, List<Long>> expected = new HashMap<String, List<Long>>() {{
+            put("TEST1.XML", Arrays.asList(76L, 66L));
+            put("TEST2.XML", Arrays.asList(81L, 76L));
+        }};
+        testInputStreamStatistics("SHRUNK.ZIP", expected);
+    }
+
+    @Test
+    public void inputStreamStatisticsForStoredEntry() throws IOException, ArchiveException {
+        Map<String, List<Long>> expected = new HashMap<String, List<Long>>() {{
+            put("test.txt", Arrays.asList(5L, 5L));
+        }};
+        testInputStreamStatistics("COMPRESS-264.zip", expected);
+    }
+
+    @Test
+    public void inputStreamStatisticsForBzip2Entry() throws IOException, ArchiveException {
+        Map<String, List<Long>> expected = new HashMap<String, List<Long>>() {{
+            put("lots-of-as", Arrays.asList(42L, 39L));
+        }};
+        testInputStreamStatistics("bzip2-zip.zip", expected);
+    }
+
+    @Test
+    public void inputStreamStatisticsForDeflate64Entry() throws IOException, ArchiveException {
+        Map<String, List<Long>> expected = new HashMap<String, List<Long>>() {{
+            put("input2", Arrays.asList(3072L, 2111L));
+        }};
+        testInputStreamStatistics("COMPRESS-380/COMPRESS-380.zip", expected);
+    }
+
+    private void testInputStreamStatistics(String fileName, Map<String, List<Long>> expectedStatistics)
+        throws IOException, ArchiveException {
+        final File input = getFile(fileName);
+
+        final Map<String,List<List<Long>>> actualStatistics = new HashMap<>();
+
+        // stream access
+        try (final FileInputStream fis = new FileInputStream(input);
+            final ArchiveInputStream in = new ArchiveStreamFactory().createArchiveInputStream("zip", fis)) {
+            for (ArchiveEntry entry; (entry = in.getNextEntry()) != null; ) {
+                readStream(in, entry, actualStatistics);
+            }
+        }
+
+        // file access
+        try (final ZipFile zf = new ZipFile(input)) {
+            final Enumeration<ZipArchiveEntry> entries = zf.getEntries();
+            while (entries.hasMoreElements()) {
+                final ZipArchiveEntry zae = entries.nextElement();
+                try (InputStream in = zf.getInputStream(zae)) {
+                    readStream(in, zae, actualStatistics);
+                }
+            }
+        }
+
+        // compare statistics of stream / file access
+        for (Map.Entry<String,List<List<Long>>> me : actualStatistics.entrySet()) {
+            assertEquals("Mismatch of stats for: " + me.getKey(),
+                         me.getValue().get(0), me.getValue().get(1));
+        }
+
+        for (Map.Entry<String, List<Long>> me : expectedStatistics.entrySet()) {
+            assertEquals("Mismatch of stats with expected value for: " + me.getKey(),
+                me.getValue(), actualStatistics.get(me.getKey()).get(0));
+        }
+    }
+
+    private void readStream(final InputStream in, final ArchiveEntry entry, final Map<String,List<List<Long>>> map) throws IOException {
+        final byte[] buf = new byte[4096];
+        final InputStreamStatistics stats = (InputStreamStatistics) in;
+        while (in.read(buf) != -1);
+
+        final String name = entry.getName();
+        final List<List<Long>> l;
+        if (map.containsKey(name)) {
+            l = map.get(name);
+        } else {
+            map.put(name, l = new ArrayList<>());
+        }
+
+        final long t = stats.getUncompressedCount();
+        final long b = stats.getCompressedCount();
+        l.add(Arrays.asList(t, b));
+    }
 }
