@@ -18,51 +18,53 @@
  */
 package org.apache.commons.compress.archivers.dump;
 
-import static org.junit.Assert.*;
+import static org.junit.Assert.assertArrayEquals;
 
 import java.io.FileInputStream;
 import java.io.InputStream;
 
 import org.apache.commons.compress.AbstractTestCase;
 import org.apache.commons.compress.archivers.ArchiveException;
-import org.junit.Test;
 
 public class DumpArchiveInputStreamTest extends AbstractTestCase {
 
-    @Test
     public void testNotADumpArchive() throws Exception {
-        try (FileInputStream is = new FileInputStream(getFile("bla.zip"))) {
+        FileInputStream is = new FileInputStream(getFile("bla.zip"));
+        try {
             new DumpArchiveInputStream(is);
             fail("expected an exception");
-        } catch (final ArchiveException ex) {
+        } catch (ArchiveException ex) {
             // expected
             assertTrue(ex.getCause() instanceof ShortFileException);
+        } finally {
+            is.close();
         }
     }
 
-    @Test
     public void testNotADumpArchiveButBigEnough() throws Exception {
-        try (FileInputStream is = new FileInputStream(getFile("zip64support.tar.bz2"))) {
+        FileInputStream is = new FileInputStream(getFile("zip64support.tar.bz2"));
+        try {
             new DumpArchiveInputStream(is);
             fail("expected an exception");
-        } catch (final ArchiveException ex) {
+        } catch (ArchiveException ex) {
             // expected
             assertTrue(ex.getCause() instanceof UnrecognizedFormatException);
+        } finally {
+            is.close();
         }
     }
 
-    @Test
     public void testConsumesArchiveCompletely() throws Exception {
-        final InputStream is = DumpArchiveInputStreamTest.class
+        InputStream is = DumpArchiveInputStreamTest.class
             .getResourceAsStream("/archive_with_trailer.dump");
-        final DumpArchiveInputStream dump = new DumpArchiveInputStream(is);
+        DumpArchiveInputStream dump = new DumpArchiveInputStream(is);
         while (dump.getNextDumpEntry() != null) {
             // just consume the archive
         }
-        final byte[] expected = new byte[] {
+        byte[] expected = new byte[] {
             'H', 'e', 'l', 'l', 'o', ',', ' ', 'w', 'o', 'r', 'l', 'd', '!', '\n'
         };
-        final byte[] actual = new byte[expected.length];
+        byte[] actual = new byte[expected.length];
         is.read(actual);
         assertArrayEquals(expected, actual);
         dump.close();
