@@ -13,7 +13,7 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- * 
+ *
  */
 
 package org.apache.commons.compress.archivers.tar;
@@ -34,7 +34,7 @@ public class TarUtilsTest {
     @Test
     public void testName(){
         byte [] buff = new byte[20];
-        String sb1 = "abcdefghijklmnopqrstuvwxyz";
+        final String sb1 = "abcdefghijklmnopqrstuvwxyz";
         int off = TarUtils.formatNameBytes(sb1, buff, 1, buff.length-1);
         assertEquals(off, 20);
         String sb2 = TarUtils.parseName(buff, 1, 10);
@@ -46,11 +46,14 @@ public class TarUtilsTest {
         assertEquals(off, 30);
         sb2 = TarUtils.parseName(buff, 1, buff.length-1);
         assertEquals(sb1, sb2);
+        buff = new byte[]{0, 1, 0};
+        sb2 = TarUtils.parseName(buff, 0, 3);
+        assertEquals("", sb2);
     }
 
     @Test
     public void testParseOctal() throws Exception{
-        long value; 
+        long value;
         byte [] buffer;
         final long MAX_OCTAL  = 077777777777L; // Allowed 11 digits
         final long MAX_OCTAL_OVERFLOW  = 0777777777777L; // in fact 12 for some implementations
@@ -82,36 +85,36 @@ public class TarUtilsTest {
         try {
             TarUtils.parseOctal(buffer,0, buffer.length);
             fail("Expected IllegalArgumentException - should be at least 2 bytes long");
-        } catch (IllegalArgumentException expected) {
+        } catch (final IllegalArgumentException expected) {
         }
         buffer=new byte[]{0}; // 1-byte array
         try {
             TarUtils.parseOctal(buffer,0, buffer.length);
             fail("Expected IllegalArgumentException - should be at least 2 bytes long");
-        } catch (IllegalArgumentException expected) {
+        } catch (final IllegalArgumentException expected) {
         }
         buffer = "abcdef ".getBytes(CharsetNames.UTF_8); // Invalid input
         try {
             TarUtils.parseOctal(buffer,0, buffer.length);
             fail("Expected IllegalArgumentException");
-        } catch (IllegalArgumentException expected) {
+        } catch (final IllegalArgumentException expected) {
         }
         buffer = " 0 07 ".getBytes(CharsetNames.UTF_8); // Invalid - embedded space
         try {
             TarUtils.parseOctal(buffer,0, buffer.length);
             fail("Expected IllegalArgumentException - embedded space");
-        } catch (IllegalArgumentException expected) {
+        } catch (final IllegalArgumentException expected) {
         }
         buffer = " 0\00007 ".getBytes(CharsetNames.UTF_8); // Invalid - embedded NUL
         try {
             TarUtils.parseOctal(buffer,0, buffer.length);
             fail("Expected IllegalArgumentException - embedded NUL");
-        } catch (IllegalArgumentException expected) {
+        } catch (final IllegalArgumentException expected) {
         }
     }
 
     private void checkRoundTripOctal(final long value, final int bufsize) {
-        byte [] buffer = new byte[bufsize];
+        final byte [] buffer = new byte[bufsize];
         long parseValue;
         TarUtils.formatLongOctalBytes(value, buffer, 0, buffer.length);
         parseValue = TarUtils.parseOctal(buffer,0, buffer.length);
@@ -136,7 +139,7 @@ public class TarUtilsTest {
     }
 
     private void checkRoundTripOctalOrBinary(final long value, final int bufsize) {
-        byte [] buffer = new byte[bufsize];
+        final byte [] buffer = new byte[bufsize];
         long parseValue;
         TarUtils.formatLongOctalOrBinaryBytes(value, buffer, 0, buffer.length);
         parseValue = TarUtils.parseOctalOrBinary(buffer,0, buffer.length);
@@ -155,18 +158,19 @@ public class TarUtilsTest {
         checkRoundTripOctalOrBinary(Long.MIN_VALUE + 1, 12);
     }
 
-    private void testRoundTripOctalOrBinary(int length) {
+    private void testRoundTripOctalOrBinary(final int length) {
         checkRoundTripOctalOrBinary(0, length);
         checkRoundTripOctalOrBinary(1, length);
         checkRoundTripOctalOrBinary(TarConstants.MAXSIZE, length); // will need binary format
         checkRoundTripOctalOrBinary(-1, length); // will need binary format
-        checkRoundTripOctalOrBinary(0xff00000000000001l, length);
+        checkRoundTripOctalOrBinary(0xffffffffffffffl, length);
+        checkRoundTripOctalOrBinary(-0xffffffffffffffl, length);
     }
 
     // Check correct trailing bytes are generated
     @Test
     public void testTrailers() {
-        byte [] buffer = new byte[12];
+        final byte [] buffer = new byte[12];
         TarUtils.formatLongOctalBytes(123, buffer, 0, buffer.length);
         assertEquals(' ', buffer[buffer.length-1]);
         assertEquals('3', buffer[buffer.length-2]); // end of number
@@ -182,20 +186,20 @@ public class TarUtilsTest {
 
     @Test
     public void testNegative() throws Exception {
-        byte [] buffer = new byte[22];
+        final byte [] buffer = new byte[22];
         TarUtils.formatUnsignedOctalString(-1, buffer, 0, buffer.length);
         assertEquals("1777777777777777777777", new String(buffer, CharsetNames.UTF_8));
     }
 
     @Test
     public void testOverflow() throws Exception {
-        byte [] buffer = new byte[8-1]; // a lot of the numbers have 8-byte buffers (nul term)
+        final byte [] buffer = new byte[8-1]; // a lot of the numbers have 8-byte buffers (nul term)
         TarUtils.formatUnsignedOctalString(07777777L, buffer, 0, buffer.length);
         assertEquals("7777777", new String(buffer, CharsetNames.UTF_8));
         try {
             TarUtils.formatUnsignedOctalString(017777777L, buffer, 0, buffer.length);
             fail("Should have cause IllegalArgumentException");
-        } catch (IllegalArgumentException expected) {
+        } catch (final IllegalArgumentException expected) {
         }
     }
 
@@ -210,22 +214,22 @@ public class TarUtilsTest {
     @Test
     public void testRoundEncoding() throws Exception {
         // COMPRESS-114
-        ZipEncoding enc = ZipEncodingHelper.getZipEncoding(CharsetNames.ISO_8859_1);
-        String s = "0302-0601-3\u00b1\u00b1\u00b1F06\u00b1W220\u00b1ZB\u00b1LALALA\u00b1\u00b1\u00b1\u00b1\u00b1\u00b1\u00b1\u00b1\u00b1\u00b1CAN\u00b1\u00b1DC\u00b1\u00b1\u00b104\u00b1060302\u00b1MOE.model";
-        byte buff[] = new byte[100];
-        int len = TarUtils.formatNameBytes(s, buff, 0, buff.length, enc);
+        final ZipEncoding enc = ZipEncodingHelper.getZipEncoding(CharsetNames.ISO_8859_1);
+        final String s = "0302-0601-3\u00b1\u00b1\u00b1F06\u00b1W220\u00b1ZB\u00b1LALALA\u00b1\u00b1\u00b1\u00b1\u00b1\u00b1\u00b1\u00b1\u00b1\u00b1CAN\u00b1\u00b1DC\u00b1\u00b1\u00b104\u00b1060302\u00b1MOE.model";
+        final byte buff[] = new byte[100];
+        final int len = TarUtils.formatNameBytes(s, buff, 0, buff.length, enc);
         assertEquals(s, TarUtils.parseName(buff, 0, len, enc));
     }
 
-    private void checkName(String string) {
-        byte buff[] = new byte[100];
-        int len = TarUtils.formatNameBytes(string, buff, 0, buff.length);
+    private void checkName(final String string) {
+        final byte buff[] = new byte[100];
+        final int len = TarUtils.formatNameBytes(string, buff, 0, buff.length);
         assertEquals(string, TarUtils.parseName(buff, 0, len));
     }
 
     @Test
     public void testReadNegativeBinary8Byte() {
-        byte[] b = new byte[] {
+        final byte[] b = new byte[] {
             (byte) 0xff, (byte) 0xff, (byte) 0xff, (byte) 0xff,
             (byte) 0xff, (byte) 0xff, (byte) 0xf1, (byte) 0xef,
         };
@@ -234,7 +238,7 @@ public class TarUtilsTest {
 
     @Test
     public void testReadNegativeBinary12Byte() {
-        byte[] b = new byte[] {
+        final byte[] b = new byte[] {
             (byte) 0xff, (byte) 0xff, (byte) 0xff, (byte) 0xff,
             (byte) 0xff, (byte) 0xff, (byte) 0xff, (byte) 0xff,
             (byte) 0xff, (byte) 0xff, (byte) 0xf1, (byte) 0xef,
@@ -245,7 +249,7 @@ public class TarUtilsTest {
 
     @Test
     public void testWriteNegativeBinary8Byte() {
-        byte[] b = new byte[] {
+        final byte[] b = new byte[] {
             (byte) 0xff, (byte) 0xff, (byte) 0xff, (byte) 0xff,
             (byte) 0xff, (byte) 0xff, (byte) 0xf1, (byte) 0xef,
         };
@@ -255,7 +259,7 @@ public class TarUtilsTest {
     // https://issues.apache.org/jira/browse/COMPRESS-191
     @Test
     public void testVerifyHeaderCheckSum() {
-        byte[] valid = { // from bla.tar
+        final byte[] valid = { // from bla.tar
                 116, 101, 115, 116, 49, 46, 120, 109, 108, 0, 0, 0, 0, 0, 0,
                 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
                 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
@@ -285,7 +289,7 @@ public class TarUtilsTest {
                 0, 0, 0, 0 };
         assertTrue(TarUtils.verifyCheckSum(valid));
 
-        byte[] compress117 = { // from COMPRESS-117
+        final byte[] compress117 = { // from COMPRESS-117
             (byte) 0x37, (byte) 0x7a, (byte) 0x43, (byte) 0x2e, (byte) 0x74, (byte) 0x78, (byte) 0x74, (byte) 0x00,
             0, 0, 0, 0, 0, 0, 0, 0,
             0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
@@ -326,7 +330,7 @@ public class TarUtilsTest {
         };
         assertTrue(TarUtils.verifyCheckSum(compress117));
 
-        byte[] invalid = { // from the testAIFF.aif file in Tika
+        final byte[] invalid = { // from the testAIFF.aif file in Tika
                 70, 79, 82, 77, 0, 0, 15, 46, 65, 73, 70, 70, 67, 79, 77, 77,
                 0, 0, 0, 18, 0, 2, 0, 0, 3, -64, 0, 16, 64, 14, -84, 68, 0, 0,
                 0, 0, 0, 0, 83, 83, 78, 68, 0, 0, 15, 8, 0, 0, 0, 0, 0, 0, 0,
@@ -360,11 +364,21 @@ public class TarUtilsTest {
 
     @Test
     public void testParseOctalCompress330() throws Exception{
-        long expected = 0100000;
-        byte [] buffer = new byte[] {
+        final long expected = 0100000;
+        final byte [] buffer = new byte[] {
             32, 32, 32, 32, 32, 49, 48, 48, 48, 48, 48, 32
         };
         assertEquals(expected, TarUtils.parseOctalOrBinary(buffer, 0, buffer.length));
     }
 
+    @Test
+    public void testRoundTripOctalOrBinary8_ValueTooBigForBinary() {
+        try {
+            checkRoundTripOctalOrBinary(Long.MAX_VALUE, 8);
+            fail("Should throw exception - value is too long to fit buffer of this len");
+        } catch (IllegalArgumentException e) {
+            assertEquals("Value 9223372036854775807 is too large for 8 byte field.", e.getMessage());
+        }
+    }
+
 }
